 /***
Purpose: Build data to run preliminary event studies estimating impact of social distancing 
			policies on behavior
Created: BG 2020-4-3
***/

*-------------------------------------------------------------------------------------------------
* Clean policy records to prepare for merge (produce a state level file and a county level file)
*-------------------------------------------------------------------------------------------------

* NOTE: Billy's "effective_date" variable incorporates the set of policies we have decided to use
*		(described in #1) with the proper prioritization order
* Rows with missing county fips correspond to state policies 
import delimited "output/full_policy_data", clear varnames(1) 
rename state st_name

* Confirm that rows with missing county_fips codes are state implimentations
assert county_fips=="NA" if geography=="state"
assert county_fips!="NA" if geography!="state"

* Convert state and county fips codes to numeric
rename state_fips state
g county=substr(county_fips, 3, 3)
destring county, replace

* Create a new date variable for each policy type
replace effective_date_sip=effective_date_sip+"/2020"
replace effective_date_business=effective_date_business+"/2020"
g date_sip=date(effective_date_sip, "MDY")
g date_bus=date(effective_date_business, "MDY")

* Create a date variable for the reopning date
replace reopen_date=reopen_date+"/2020" if reopen_date!="NA"
g date_reopen=date(reopen_date, "MDY")
format date_sip date_bus date_reopen %td

* Output a data set of reopening dates at the statelevel
preserve
keep if ~mi(date_reopen)
keep state date_reopen
isid state
tempfile reopen_policies
save `reopen_policies'
restore

* Keep the earliest stay at home order in each state county
collapse (min) date_sip date_bus, by(state county)

* Output a tempfile of state policies and county policies for a later merge
preserve
keep if mi(county)
drop county
rename (date_sip date_bus) (st_date_sip st_date_bus)
tempfile st_policies
save `st_policies'
restore 

* County file
keep if ~mi(county)
rename (date_sip date_bus) (cty_date_sip cty_date_bus)
tempfile cty_policies
save `cty_policies'

*-------------------------------------------------------------------------------------------------
* Most updated cases and deaths
*-------------------------------------------------------------------------------------------------

import delimited "output/county_case_death_data.csv", clear varnames(1)

destring county_fips, gen(fips) force

* Format the date variable
rename date tmp
g date=date(tmp, "YMD")
format date %td

* Tempfile
drop if mi(fips)
assert ~mi(cases) & ~mi(deaths)
keep fips date cases deaths 
compress
tempfile covid
save `covid'

*-------------------------------------------------------------------------------------------------
* Hospitlizations
*-------------------------------------------------------------------------------------------------

* States get a row once any variable becomes present
import delimited "output/state_hospitalizations.csv", clear varnames(1)

* Clean date variable
rename date tmp
g tmp2=tmp+"/2020"
g date=date(tmp2, "MDY")
assert ~mi(date)
format date %td
drop tmp tmp2 
xtset state_fips date

* Hospitilizations
destring hospitalizations, replace force

* Tests
destring tests, replace force

* Output tempfile
keep state_fips date tests hospitalizations
rename state_fips state
tempfile tests
save `tests'


*-------------------------------------------------------------------------------------------------
* Flights
*-------------------------------------------------------------------------------------------------
import delimited "output/daily_flight_counts.csv", clear varnames(1)
keep if day < "2020-03-01"
gen international = (destination_country == "United States" & origin_country != "United States")
keep if international == 1
collapse (sum) flight_count, by(destination_geography)
rename destination_geography fips
destring fips, replace
tempfile flights
save `flights'

*-------------------------------------------------------------------------------------------------
* Merge on policies and other variables to SafeGraph data
*-------------------------------------------------------------------------------------------------

import delimited "input/county_day_data.csv", clear
isid fips date
rename state_fips state 

* Generate a county fips
tostring fips, format(%05.0f) gen(tmp)
g county=substr(tmp, 3, 3)
destring county, replace
drop tmp

* Convert string data to stata formatted date
g tmp=date(date, "YMD")
drop date
rename tmp date
format date %td
assert ~mi(date)
xtset fips date
assert "`r(balanced)'"=="strongly balanced"

* Merge flight data
merge m:1 fips using `flights', keep(1 3) nogen 
replace flight_count = 0 if flight_count == .
rename flight_count inbound_international_flights

* Merge on cases and deaths + tests and hospitalizations
merge 1:1 fips date using `covid', keep(1 3) nogen
merge m:1 state date using `tests', keep(1 3) nogen

* Set health variables to 0 if missing and define non-cumulativie versions
sort fips date
foreach h in cases deaths tests hospitalizations {
	replace `h'= 0 if missing(`h')
	g new_`h'=`h'-l.`h'
}

* Merge on the state and county policies
* Non-matching counties/states are for places like puerto rico in the state merge
merge m:1 state using `st_policies', keep(1 3) nogen
merge m:1 state county using `cty_policies', keep(1 3) nogen

* Create an indicator for whether the county currently has a stay at home policy 
g order_sip=(date>=st_date_sip & ~mi(st_date_sip)) | (date>=cty_date_sip & ~mi(cty_date_sip))
g order_bus=(date>=st_date_bus & ~mi(st_date_bus)) | (date>=cty_date_bus & ~mi(cty_date_bus))

* Date of first order for each county
g tmp=date if order_sip==1
egen first_day_order_sip=min(tmp), by(fips)
drop tmp
g tmp=date if order_bus==1
egen first_day_order_bus=min(tmp), by(fips)
drop tmp 
format first_day_order* %td 

* Merge on the state level reopening policies
merge m:1 state using `reopen_policies', keep(1 3) nogen

* Create an indicator for whether the county is "open"
* 3 conditions under which this indicator is 1
* You have a reopening date and the current day is on or after the reopening
*	you have no reopening date and you do not currently have a SIP or BUS order
*	you have a reopening order but the current date is before the SIP or BUS orders
*	(last case should cover cases with missing bis and sip dates via missing)
g state_is_open=(date>=date_reopen & ~mi(date_reopen)) | ///
	(mi(date_reopen) & order_sip==0 & order_bus==0) | ///
	(date<first_day_order_sip & date<first_day_order_bus & ~mi(date_reopen))
	
* Set the sip and business orders to 0 after the reopening date
replace order_sip=0 if date>=date_reopen & ~mi(date_reopen)
replace order_bus=0 if date>=date_reopen & ~mi(date_reopen)

* Merge on debit card transactions
* 95% match rate and 99.9% count weighted match rate
merge 1:1 fips date using "output/facteus_county_daily", keep(1 3) nogen

* Merge on homebase data
merge 1:1 fips date using "output/homebase_by_county_cleaned", keep(1 3) nogen

* Create an indicator for county being above the 25th percentile in terms of fraction of pop covered
*	by homebase
g tmp=count_hours_pos/pop if inrange(date, td(1feb2020), td(7feb2020))
egen homebase_coverage=mean(tmp), by(fips)
drop tmp

* Merge on state and county names
g cty1990=fips
merge m:1 cty1990 using "input/online_table4-2", ///
	keep(1 3) keepusing(county_name statename stateabbrv) nogen
drop cty1990

* Merge on the county covariates 
merge m:1 fips using "output/cty_covariates_with_region.dta", keep(1 3) nogen ///
	keepusing(adjmortmeas_pnall30day adjmortmeas_amiall30day cur_smoke_q1 bmi_obese_q1 exercise_any_q1 ///
	region)

* Merge on CBSA
merge m:1 fips using "output/county_cbsa", keep(1 3) nogen
	
*-------------------------------------------------------------------------------------------------
* Final cleaning for event study setup
*-------------------------------------------------------------------------------------------------

* Generate logs of outcome variables
foreach y in visits sum_hours total_spend num_transactions sum_wages count_hours_pos {
	g log_`y'=log(`y')
}

* Date at which you had 0.01% of your population with cases
g tmp=date if (cases/pop)>.0001 & ~mi(cases) & ~mi(pop)
egen date_virus_arrives=min(tmp), by(fips)

* Add event study variables
* Define event time within each county by storing the date of the first order
g event_day=date-first_day_order_sip

* Set treatment variable -- those with missing event times never got designated
g treatment=~mi(event_day)

* Make a business closing set of variables (as of now we are not using these)
g event_day_bus=date-first_day_order_bus
g treatment_bus=~mi(event_day_bus)
	

* Keep a limited number of variables 
rename (state county) (state_fips county_fips)
order fips county_name statename stateabbrv date order_sip order_bus first_day_order_sip first_day_order_bus visits log_visits cases deaths new_cases new_deaths inbound_international_flights
sort fips date 
compress
save "output/event_study_input_data", replace
outsheet using "output/event_study_input_data.csv",  replace comma

*-------------------------------------------------------------------------------------------------
* CSA level data 
*-------------------------------------------------------------------------------------------------
* Get denom variables
import delimited "input/county_demographics.csv", clear
keep county *_denom pop_occd* pop_urban summer_tmmx winter_tmmx pm2
rename county fips
tempfile denom
save `denom'

* Loop over the business and SIP data and produce a CSA level file
* "both" produces a data set only containing counties that had a SIP and BIS on same day
foreach i in sip bus both open {
	use if ~mi(csacode) using "output/event_study_input_data", clear
	
	merge m:1 fips using `denom', keep(1 3) nogen 
	merge m:1 fips using "output/cty_covariates_with_region", keep(1 3) nogen ///
				keepusing(hhinc_mean2000 med_hhinc2016 gsmn_math_g3_2013 rent_twobed2015 ///
				ann_avg_job_growth_2004_2013 job_density_2013 region division)

	replace cases     = 0 if cases     == .
	replace deaths    = 0 if deaths    == .
	replace new_tests = 0 if new_tests == .
	
	* Assign the policy variable 
	local o `i'
	if "`i'"=="both" {
		keep if first_day_order_sip==first_day_order_bus & ~mi(first_day_order_sip)
		local o sip
	}
	if "`i'"=="open" {
		* Restrict to counties that had a SIP as of April 5 and then keep data from April 5 onwards
		keep if first_day_order_sip<=td(5apr2020) & date>=td(5apr2020)

		* Define event time within each county by storing the reopening date
		rename date_reopen first_day_order_open
	}

	* Collapse to the CSA by order timing by day level
	destring summer_tmmx winter_tmmx pm2, force replace
	collapse (rawsum) visits cases deaths new_tests pop pop_* land_area_sq_km inbound_international_flights ///
						sum_hours total_spend num_transactions sum_wages count_hours_pos ///
			(mean) r_share summer_tmmx winter_tmmx pm2 ///
					hhinc_mean2000 med_hhinc2016 gsmn_math_g3_2013 rent_twobed2015 ///
					ann_avg_job_growth_2004_2013 job_density_2013 ///
			[w=pop], by(csacode csatitle first_day_order_`o' date)
		
	* Generate logs of outcome variables
	foreach y in visits sum_hours total_spend num_transactions sum_wages count_hours_pos {
		g log_`y'=log(`y')
	}
		

	* Define event time within each county by storing the date of the first order
	g event_day=date-first_day_order_`o'

	* Set treatment variable -- those with missing event times never got designated
	g treatment=~mi(event_day)
	
	* Make csacode order date bins
	gen timing_group = first_day_order_`o'
	replace timing_group = 0 if timing_group == .
	egen csa_order_date = group(csacode timing_group)
	drop timing_group
	
	* Make change in visits variables
	bysort csa_order_date: gen temp = visits if date == mdy(3, 1, 2020)
	bysort csa_order_date: egen base_visits = max(temp)
	drop temp
	gen base_log_visits  = log(base_visits)
	gen delta_log_visits = log_visits - base_log_visits
	
	* Make lag variables
	xtset csa_order_date date
	foreach v in cases deaths {
		replace `v'  = 0 if `v' == .
		replace `v'  = l.`v' if `v' < l.`v' & l.`v' != .
		gen l_`v'    = l.`v'
		gen f_`v'    = f.`v'
	}
	
	* Estimation Sample
	bysort csa_order_date: egen max_rel_day = max(event_day * (cases >= 10))
	bysort csa_order_date: egen min_rel_day = min(event_day * (cases >= 10))
	gen estimation_sample = 0
	replace estimation_sample = 1 if (max_rel_day == . |  ((min_rel_day <= -8) & (max_rel_day >= 20))) & cases >= 10 & (date >= mdy(2, 1, 2020) & date < mdy(5, 1, 2020))
	
	* Create model structure variables (t is indexed relative to the estimation sample)
	xtset csa_order_date date
	bysort csa_order_date: gen t = _n if estimation_sample == 1
	bysort csa_order_date: egen min_t = min(t)
	replace t = t - min_t + 1
	drop min_t
	cap drop I_t*
	
	local gamma_0  = 0
	local gamma_12 = 1/12
	local gamma_9  = 1/9
	local gamma_6  = 1/6
	local gamma_3  = 1/3
	local gamma_1  = 1
	gen I_t_0  = cases if t == 1
	gen I_t_12 = cases if t == 1
	gen I_t_9  = cases if t == 1
	gen I_t_6  = cases if t == 1
	gen I_t_3  = cases if t == 1
	gen I_t_1  = cases if t == 1

	levelsof t, local(levels)
	foreach x of local levels {
		if `x' != 1 {
			foreach g in 0 12 9 6 3 1 {
				replace I_t_`g' = (cases - l_cases) + (1 - `gamma_`g'') * l.I_t_`g'	 if t == `x'
			}
		}
	}

	foreach g in 0 12 9 6 3 1{
		gen log_I_t_`g'       = log(I_t_`g')
	}

	gen log_new_cases  = log(.5 + f.cases - cases)
	gen log_new_deaths = log(.5 + f.deaths - deaths)
	gen log_cases      = log(cases)
	
	* Create event windows
	xtset csa_order_date date
	gen pre_event_window     = 0
	replace pre_event_window = 1 if (event_day <= -9 & event_day != .)
	gen event_window     = 0
	replace event_window = 1 if (event_day >= -8 & event_day <= 20 & event_day != .)
	gen post     = 0
	replace post = 1 if (event_day >= 0 & event_day <= 20 & event_day != .)
	gen post_post     = 0
	replace post_post = 1 if (event_day >= 21 & event_day != .)

	foreach e in 7 14 21 {
		gen event_window_`e' = 0
		replace event_window_`e' = 1 if (event_day >= -(`e' - 1) & event_day <= 20 & event_day != .)	
	}
	
	* Output the data 
	compress
	order csacode date
	sort csacode date
	save "output/csa_`i'_event_study_input_data", replace
} 
