library(dplyr)
library(readr)
library(usmap)
library(stringr)
library(tidyr)
library(gsheet)
library("rjson")


## If we want to update the pulled policy data, run '/code/pull_policy_data.R'

nyt_policies <- read_csv('input/nyt_state_local_policies_raw.csv')
nyt_reopen <- read_csv('input/nyt-states-reopen-status-covid-19_dataset.csv')
city_counties <- read_csv('input/city_counties_raw.csv')
keystone_raw <- read_csv('input/keystone_raw.csv')
crowdsource_raw <- read_csv('input/crowdsource_raw.csv')
hikma_json <- readRDS('input/hikma_json.rds')
county_cbsa_xw_raw <- read_csv('input/cbsa2fipsxw.csv')

### Clean nyt data
nyt_policies %>%
  mutate(state_fips = fips(state)) -> nyt_policies

nyt_policies %>%
  filter(geography == 'county') %>%
  rowwise %>%
  mutate(county_fips = fips(state, county = substate)) %>%
  select(substate, state_fips, county_fips) -> nyt_county_fips

nyt_policies %>%
  filter(geography == 'city') %>%
  select(state, state_fips, 'city' = substate) %>% 
  left_join(city_counties, by = c('state', 'city')) %>% 
  rowwise %>%
  mutate(county_fips = fips(state, county = county)) %>% 
  select('substate' = city, state_fips, county_fips) -> city_county_fips

dc_fips <- data.frame(substate = rep(NA, 2), state_fips = rep('11', 2), county_fips = c('11001', NA))

nyt_policies %>%
  left_join(rbind(nyt_county_fips, city_county_fips, dc_fips), 
            by = c('substate', 'state_fips')) %>%
  rename(nyt_effective_date = effective_date)-> nyt_policies

nyt_policies %>%
  filter(state != 'PR') %>%
  select(state_fips, county_fips, nyt_effective_date) %>%
  mutate(nyt_effective_date = as.numeric(paste0(substr(nyt_effective_date, 1, 2), 
                                                substr(nyt_effective_date, 4, 5)))) %>%
  group_by(state_fips, county_fips) %>%
  summarize(nyt_effective_date = min(nyt_effective_date)) %>%
  mutate(nyt_effective_date = paste0('0', substr(nyt_effective_date, 1, 1), 
                                     '/', substr(nyt_effective_date, 2, 3))) -> nyt_policies_for_joining

nyt_reopen %>%
  filter(state != 'Puerto Rico') %>%
  select(state, restriction_start, reopen_date = restriction_end) %>%
  mutate(reopen_date = ifelse(is.na(reopen_date), NA, 
                              paste0(substr(reopen_date, 6, 7), '/', 
                                     substr(reopen_date, 9, 10))) )%>%
  rowwise %>% mutate(geography = 'state') %>% ungroup -> nyt_reopen_for_joining

# manual NYT reopen date info
manual_reopen <- data.frame(state = c('Kentucky', 'New Jersey', 'Wisconsin', 'Tennessee'),
                            geography = rep('state', 4),
                            manual_reopen_date = c('05/20', '06/09', '05/13', NA))

# some states are having regional reopenings based on county information 
# 50/58 California Counties 
# 35/36 Colorado Counties 
# 89/95 Tennessee Counties 
regional_reopen <- data.frame(state = c('California', 'Oregon', 'Tennessee'),
                              geography = rep('state', 3),
                              regional_reopen_date = c('05/25', '06/05', '04/29'))

#################### Clean keystone data ############################


bad_date_formats <- c('24-Mar', '22-Mar', '17-Mar', '16-Mar')
variations_on_na <- c('None in Place', 'start', 'None in place', 'none in place')

keystone_raw %>%
  mutate(fip_code = ifelse(nchar(fip_code) < 4, 
                           str_pad(fip_code, 2, side = 'left', pad = '0'), 
                           str_pad(fip_code, 5, side = 'left', pad = '0'))) %>%
  mutate(state_fips = substr(fip_code, 1, 2)) %>%
  mutate(county_fips = ifelse(nchar(fip_code) == 2, NA, fip_code)) %>%
  # fix dates that should be NA 
  mutate(start_date = ifelse(start_date %in% variations_on_na, NA, start_date)) %>%
  # fix bad date formats
  mutate(start_date = ifelse(start_date == '24-Mar', '03/24/2020', start_date)) %>%
  mutate(start_date = ifelse(start_date == '22-Mar', '03/22/2020', start_date)) %>%
  mutate(start_date = ifelse(start_date == '17-Mar', '03/17/2020', start_date)) %>%
  mutate(start_date = ifelse(start_date == '16-Mar', '03/16/2020', start_date)) %>%
  separate(start_date, into = c('month', 'date', 'year'), sep = '/') %>%
  mutate(effective_date = ifelse(is.na(month), NA, 
                                 paste0(str_pad(month, 2, side = 'left', pad = '0'), '/', 
                                        str_pad(date, 2, side = 'left', pad = '0')))) %>%
  mutate(geography = ifelse(is.na(county), 'state', 'county')) %>%
  select(state, substate = county, state_fips, county_fips, geography, npi, effective_date) %>%
  spread(key = npi, value = effective_date) -> new_keystone


#################### Clean crowdsourced Bernstein data ############################

crowdsource_raw %>%
  select(state, substate = county, 
         crwd_business_closed = business_closed_date, 
         crwd_school_closed = school_closed_date,
         crwd_college_closed = college_closed_date, 
         crwd_religion_closed = religion_closed_date,
         crwd_lockdown = lockdown_closed_date) %>%
  # sacramento is spelled wrong in the crowdsourced data
  mutate(substate = as.character(ifelse(substate == 'Sacremento County', 'Sacramento County', substate))) %>% 
  mutate(substate = as.character(ifelse(substate == 'St Charles County', 'St. Charles County', substate))) %>% 
  filter(state != 'Puerto Rico') %>%
  mutate(substate = as.character(ifelse(substate == 'Yes', NA, substate))) %>%
  mutate(crwd_business_closed = paste0(substr(crwd_business_closed, 6, 7),
                                       substr(crwd_business_closed, 9, 10))) %>%
  mutate(crwd_school_closed = paste0(substr(crwd_school_closed, 6, 7), 
                                     substr(crwd_school_closed, 9, 10))) %>%
  mutate(crwd_college_closed = paste0(substr(crwd_college_closed, 6, 7), 
                                      substr(crwd_college_closed, 9, 10))) %>%
  mutate(crwd_religion_closed = paste0(substr(crwd_religion_closed, 6, 7), 
                                       substr(crwd_religion_closed, 9, 10))) %>%
  mutate(crwd_lockdown = paste0(substr(crwd_lockdown, 6, 7), 
                                substr(crwd_lockdown, 9, 10))) %>%
  gather(key = 'policy', value = 'effective_date', -state, -substate) %>%
  mutate(effective_date = ifelse(effective_date == 'NANA', NA, effective_date)) %>%
  rowwise %>%
  mutate(state_fips = fips(state)) %>%
  mutate(county_fips = ifelse(is.na(substate), NA, fips(state, substate))) %>%
  mutate(geography = ifelse(is.na(substate), 'state', 'county')) %>%
  mutate(state = as.character(fips_info(state_fips)$abbr)) %>%
  ungroup %>%
  mutate(effective_date = as.numeric(effective_date)) %>%
  group_by(state, substate, policy, state_fips, county_fips, geography) %>%
  summarize(effective_date = min(effective_date)) %>%
  mutate(effective_date = ifelse(is.na(effective_date), NA, 
                                 paste0('0', substr(effective_date, 1, 1), 
                                        '/', substr(effective_date, 2, 3)))) %>%
  spread(key = policy, value = effective_date) -> clean_crowdsource

clean_crowdsource %>%
  ungroup %>%
  select(-one_of(c('state', 'substate', 'geography'))) -> crwd_for_join


################ hikma data ####################################################################################################
clean_hikma_json <- function(index) {
  hikma_obs <- data.frame(matrix(hikma_json[[index]], nrow = 1))
  colnames(hikma_obs) <- names(hikma_json[[index]])
  return(hikma_obs)
}

hikma_counties <- names(hikma_json)

bind_rows(lapply(seq(1:length(hikma_json)), clean_hikma_json), 
          .id = "column_label") %>%
  ungroup %>% 
  mutate(column_label = hikma_counties) %>%
  rename(county_fips = column_label) %>%
  select(county_fips, contains('date')) %>%
  select(-updated) %>% 
  gather(key = policy, value = effective_date, -county_fips) %>%
  rowwise %>%
  mutate(effective_date = ifelse(is.null(effective_date),
                                 NA,
                                 paste0(substr(effective_date, 6, 7), '/',
                                        substr(effective_date, 9, 10)))) %>%
  mutate(policy = paste0('hikma_', policy)) %>%
  spread(key = policy, value = effective_date) %>%
  mutate(county_fips = as.character(ifelse(str_length(county_fips) == 5, 
                                           county_fips, 
                                           paste0('0', county_fips)))) %>%
  mutate(state_fips = substr(county_fips, 1, 2)) %>%
  mutate(geography = 'county') -> hikma

hikma %>%
  select(-geography) -> hikma_for_join

##################### crosswalk cleaning ######################################
county_cbsa_xw_raw %>%
  filter(!is.na(cbsacode)) %>%
  mutate(county_fips = paste0(fipsstatecode, fipscountycode)) %>%
  mutate(is_metro = as.integer(str_detect(metropolitanmicropolitanstatis, 'Metro'))) %>% 
  select(county_fips, cbsacode, cbsaname = cbsatitle, is_metro) -> county_cbsa_xw

############# MAKE FULL POLICY DATASET ##########################

clean_vars <- c('state', 'substate', 'state_fips', 'county_fips', 'geography', 
                'cbsacode', 'cbsaname', 'is_metro', 'effective_date_sip', 
                'source_sip', 'effective_date_business', 'source_business', 
                'effective_date_school', 'source_school', 'reopen_date')

new_keystone %>%
  full_join(nyt_policies_for_joining, by = c('state_fips', 'county_fips')) %>%
  full_join(crwd_for_join, by = c('state_fips', 'county_fips')) %>%
  full_join(hikma_for_join, by = c('county_fips', 'state_fips')) %>%
  left_join(nyt_reopen_for_joining, by = c('state', 'geography')) %>%
  left_join(county_cbsa_xw, by = 'county_fips') %>%
  # HERES WHERE WE DECIDE HOW TO RELATE THE POLICIES 
  mutate(effective_date_sip = ifelse(!is.na(nyt_effective_date), nyt_effective_date,
                                     ifelse(!is.na(shelter_in_place), shelter_in_place, 
                                            ifelse(!is.na(crwd_lockdown), crwd_lockdown,
                                                   hikma_shelter_date)))) %>%
  mutate(source_sip = ifelse(!is.na(nyt_effective_date), 'nyt', 
                             ifelse(!is.na(shelter_in_place), 'keystone_sip', 
                                    ifelse(!is.na(crwd_lockdown), 'crwd_lockdown',
                                           ifelse(!is.na(hikma_shelter_date), 'hikma_shelter', NA))))) %>%
  mutate(effective_date_business = ifelse(!is.na(closing_of_public_venues), closing_of_public_venues, 
                                          ifelse(!is.na(crwd_business_closed), crwd_business_closed, 
                                                 ifelse(!is.na(hikma_work_date), hikma_work_date,
                                                        effective_date_sip)))) %>%
  mutate(source_business = ifelse(!is.na(closing_of_public_venues), 'keystone_cpv', 
                                  ifelse(!is.na(crwd_business_closed), 'crwd_business',
                                         ifelse(!is.na(hikma_work_date), 'hikma_work',
                                                'effective_date_sip')))) %>%
  mutate(effective_date_school = ifelse(!is.na(school_closure), school_closure, 
                                          ifelse(!is.na(crwd_school_closed), crwd_school_closed, 
                                                 ifelse(!is.na(hikma_school_date), hikma_school_date,
                                                        effective_date_sip)))) %>%
  mutate(source_school = ifelse(!is.na(school_closure), 'keystone_school_closure', 
                                  ifelse(!is.na(crwd_school_closed), 'crwd_school',
                                         ifelse(!is.na(hikma_school_date), 'hikma_school',
                                                'effective_date_sip')))) %>%
  filter((!is.na(effective_date_sip) | !is.na(effective_date_business) | !is.na(effective_date_school))) %>%
  # here we deal with exceptions for the state policies verified with help from 
  # https://www.cnn.com/2020/03/23/us/coronavirus-which-states-stay-at-home-order-trnd/index.html
  # Utah never had a SIP order (though they had a loose stay at home order) 
  # NOTE: Oklahoma has a "Safer at home" order" but only applies to select business and vulnerable populations
  # Tennessee issued a soft order on 3/31 from NYT data, but the strict order is 4/02
  # https://www.tennessean.com/story/news/politics/2020/04/20/tennessee-gov-wont-extend-coronavirus-home-order/5166816002/
  mutate(effective_date_sip = ifelse(state == 'Utah' & geography == 'state', NA,
                                     effective_date_sip),
         source_sip = ifelse(state == 'Utah' & geography == 'state', NA,
                             source_sip)) %>%
  mutate(effective_date_sip = ifelse(state == 'Tennessee' & geography == 'state', '04/02',
                                     effective_date_sip),
         source_sip = ifelse(state == 'Tennessee' & geography == 'state', 'manual',
                             source_sip)) %>%
  select(clean_vars)-> full_policy_data

# include manual repopen date information 
full_policy_data %>%
  left_join(manual_reopen, by = c('state', 'geography')) %>%
  left_join(regional_reopen, by = c('state', 'geography')) %>% 
  mutate(reopen_date = ifelse(state %in% manual_reopen$state, as.character(manual_reopen_date), reopen_date)) %>%
  mutate(reopen_date_no_regional = reopen_date) %>%
  mutate(reopen_date = ifelse(is.na(reopen_date), as.character(regional_reopen_date), reopen_date)) %>%
  select(-manual_reopen_date, -regional_reopen_date) -> full_policy_data


write_csv(full_policy_data, 'output/full_policy_data.csv')

