library(dplyr)
library(readr)
library(usmap)

state_raw <-read_csv("input/covidtracking_daily.csv")

state_raw %>%
  rowwise %>%
  # make date usable for calculations
  mutate(date = substr(date, 5, 8)) %>%
  mutate(state_fips = fips(state)) %>%
  # filter out islands and territories
  filter(!is.na(state_fips)) %>%
  # define our preferred hospitalizations variable as some states
  # report current hospitalizations and not cumulative
  # when cumulative is not available, we use current hospitalizations
  mutate(hospitalizations = ifelse(is.na(hospitalizedCumulative),
                                         hospitalizedCurrently,
                                         hospitalizedCumulative),
         imputed_with_current = ifelse(is.na(hospitalizedCumulative) && !is.na(hospitalizedCurrently),
                                       1,
                                       0)) -> intermediate_data

# get the first date we see available hospitalization data
  intermediate_data %>%
    select(state, date, hospitalizations) %>% 
    filter(!is.na(hospitalizations)) %>%
    group_by(state) %>%
    summarize(first_hospitalization_date = min(date)) -> first_data_dates
  
# function that returns the most recent hospitalization data that exists
  impute_from_previous <- function(data, curr_state, curr_date) {
    data %>%
      filter(!is.na(hospitalizations)) %>%
      filter(state == curr_state) %>%
      filter(date < curr_date) %>%
      group_by(state) %>%
      filter(date == max(date)) %>%
      ungroup %>%
      select(hospitalizations) %>% unlist -> imputed_hospitalizations
    
    return(imputed_hospitalizations)
  }

# for states that go in and out of the sample, impute NAs with most recent hospitalization data
  intermediate_data %>%
    left_join(first_data_dates, by = 'state') %>%
    ungroup %>%
    rowwise %>%
    filter(is.na(hospitalizations)) %>%
    filter(date > first_hospitalization_date) %>%
    select(date, state) %>%
    mutate(imputed_previous_hospitalizations = impute_from_previous(intermediate_data,
                                                                    state,
                                                                    date)) %>%
    mutate(imputed_with_previous = 1) -> impute_with_previous_values
  
# put everything together and clean
intermediate_data %>%
  left_join(first_data_dates, by = 'state') %>%
  left_join(impute_with_previous_values, by = c('state', 'date')) %>%
  #  variable notes that hospitalizations were imputed with most recent available data
  mutate(imputed_with_previous = ifelse(is.na(imputed_with_previous), 0, 1)) %>%
  mutate(hospitalizations = ifelse(is.na(hospitalizations),
                                   imputed_previous_hospitalizations,
                                   hospitalizations)) %>%
  mutate(date = paste0(substr(date, 1, 2), '/', substr(date, 3, 4))) %>%
  mutate(first_hospitalization_date = paste0(substr(first_hospitalization_date, 1, 2), 
                                             '/', 
                                             substr(first_hospitalization_date, 3, 4))) %>%
  select(date, state, state_fips, hospitalizations,
         imputed_with_current, imputed_with_previous,
         tests = totalTestResults,
         cases = positive,
         negative_tests = negative,
         pending_tests = pending, 
         current_hospitalizations = hospitalizedCurrently,
         cumulative_hospitalizations = hospitalizedCumulative,
         current_icu = inIcuCurrently,
         cumulative_icu = inIcuCumulative,
         current_ventilator = onVentilatorCurrently,
         cumulative_ventilator = onVentilatorCumulative,
         recovered_cases = recovered) -> state_hospitalizations

#NOTE: 'hospitalizations' is the preferred number

write_csv(state_hospitalizations, 'output/state_hospitalizations.csv')


