import numpy as np
import pandas as pd
import geopandas as gp

# the derived data airport_data.csv is available on Dropbox.
# to produce airport_data.csv from scratch, run the script 
# data/code/get_airports_data.py
airports_data = pd.read_csv('external/airports_data.csv', encoding = 'utf-8')

# load county shape files 
county_shapes = gp.read_file('input/county_shapefiles')
county_shapes['county_fips'] = county_shapes['STATE'] + county_shapes['COUNTY']
county_shapes = county_shapes[['county_fips', 'geometry']]

# get U.S. airport data and set-up to attach county_fips
us_airports = airports_data[airports_data.country == 'United States']
us_airports = us_airports[['name', 'icao', 'latitude', 'longitude', 'country', 'municipality']]
us_airports = gp.GeoDataFrame(us_airports, 
                              geometry=gp.points_from_xy(us_airports.longitude, 
                                                         us_airports.latitude))
us_airports.crs = {'init': 'epsg:4269'}

# merge airport location with county shapefiles to get county_fips
us_county_fips = gp.sjoin(us_airports, county_shapes, how = 'left', op = 'within')
us_county_fips = us_county_fips.loc[pd.notnull(us_county_fips.county_fips)]
us_county_fips = us_county_fips[['icao', 'county_fips', 'country']]
us_county_fips.rename(columns = {'county_fips' : 'geography'})

# set up international airports info
inter_airports = airports_data[airports_data.country != 'United States']
inter_airports = inter_airports[['icao', 'country']]
inter_airports['geography'] = inter_airports['country']

# create master crosswalks from IACO airport code to both country and desired geography (county if U.S.)
geography_xw = us_county_fips.rename(columns = {'county_fips' : 'geography'}).append(inter_airports)

# set up destination and origin airport data to join into the routes data
destination_airports = airports_data[['name', 'icao', 'latitude', 'longitude', 'country', 'municipality']]
destination_airports = destination_airports.rename(columns = {'name' : 'destination_airport_name',
                                                              'icao' : 'destination_icao',
                                                              'latitude' : 'destination_lat', 
                                                              'longitude': 'destination_long',
                                                              'country' : 'destination_country',
                                                              'municipality' : 'destination_city'})

origin_airports = airports_data[['name', 'icao', 'latitude', 'longitude', 'country', 'municipality']]
origin_airports = origin_airports.rename(columns = {'name' : 'origin_airport_name',
                                                    'icao' : 'origin_icao',
                                                    'latitude' : 'origin_lat', 
                                                    'longitude': 'origin_long',
                                                    'country' : 'origin_country',
                                                    'municipality' : 'origin_city'})


# load raw flight data by day 
# Source: OpenSky Network; url: https://zenodo.org/record/3928550#.XxY4XpNKh25
dec_routes = pd.read_csv('external/flightlist_122019.csv', encoding = 'utf-8')
jan_routes = pd.read_csv('external/flightlist_012020.csv', encoding = 'utf-8')
feb_routes = pd.read_csv('external/flightlist_022020.csv', encoding = 'utf-8')
mar_routes = pd.read_csv('external/flightlist_032020.csv', encoding = 'utf-8')
apr_routes = pd.read_csv('external/flightlist_042020.csv', encoding = 'utf-8')
may_routes = pd.read_csv('external/flightlist_052020.csv', encoding = 'utf-8')
jun_routes = pd.read_csv('external/flightlist_062020.csv', encoding = 'utf-8')

# combine all months and remove observations where we don't observe origin, destination, and day
all_routes = pd.concat([dec_routes, jan_routes, feb_routes, mar_routes, apr_routes, may_routes, jun_routes])
all_routes = all_routes[['origin', 'destination', 'day']]
all_routes = all_routes.loc[pd.notnull(all_routes.origin)]
all_routes = all_routes.loc[pd.notnull(all_routes.destination)]

# join destination and origin airport information to each route
all_routes = pd.merge(all_routes, destination_airports, how = 'inner', 
                  left_on = 'destination', right_on = 'destination_icao')
all_routes = pd.merge(all_routes, origin_airports, how = 'inner', 
                  left_on = 'origin', right_on = 'origin_icao')

# clean day format
all_routes['day'] = all_routes['day'].astype(str).str[:10]

# merge geography (county_fips) information
all_routes = pd.merge(all_routes, geography_xw, how = 'inner', 
                     left_on = 'origin_icao', right_on = 'icao')
all_routes = all_routes.rename(columns = {'geography': 'origin_geography'})
all_routes = pd.merge(all_routes, geography_xw, how = 'inner', 
                     left_on = 'destination_icao', right_on = 'icao')
all_routes = all_routes.rename(columns = {'geography': 'destination_geography'})
all_routes = all_routes.drop(['icao_x', 'icao_y', 'country_x', 'country_y'], axis = 1)

# write dataframe listing all flights
# huge file, so we will not write it to csv by default
# all_routes.to_csv('../../data/output/all_flights.csv')

# count number of flights each day for each country X county combo
group_counts = all_routes[['day', 'origin_geography', 
                           'destination_geography', 'origin']].groupby(['day', 'origin_geography', 
                                                                        'destination_geography']).count()
group_counts = group_counts.rename(columns = {'origin' : 'flight_count'})
group_counts = group_counts.reset_index()

# add origin and destination countries to grouped counts
group_counts['origin_country'] = group_counts['origin_geography']
group_counts.loc[group_counts.origin_country.isin(us_county_fips.county_fips), 
                 'origin_country'] = 'United States'
group_counts['destination_country'] = group_counts['destination_geography']
group_counts.loc[group_counts.destination_country.isin(us_county_fips.county_fips), 
                 'destination_country'] = 'United States'

# write dataframe listing flight count each day for each origin X destination combo
group_counts.to_csv('output/daily_flight_counts.csv', index = False, encoding = 'utf-8')