library(dplyr)
library(readr)
library(stringr)
library(tidyr)

write_gslab_table <- function(TABLE, output, header){
  write.table(TABLE, file = output, row.names = FALSE, col.names = FALSE, sep = "\t")
  table <- read_file(output) # http://stackoverflow.com/questions/9068397/import-text-file-as-single-character-string
  table <- sprintf("%s\n%s", header, gsub('"', '', table))
  write_file(table, output)
}

event_study_data <- read_csv('input/event_study_input_data.csv') 
raw_policy_data  <- read_csv('input/full_policy_data.csv')


event_study_data %>%
  distinct(fips, county_name, stateabbrv, first_day_order_sip,
           first_day_order_bus, pop) %>%
  mutate(final_sip = paste0(ifelse(substr(first_day_order_sip, 3, 5) == 'mar',
                                   '03/', '04/'),
                            substr(first_day_order_sip, 1, 2))) %>%
  mutate(final_bus = paste0(ifelse(substr(first_day_order_bus, 3, 5) == 'mar',
                                   '03/', '04/'),
                            substr(first_day_order_bus, 1, 2))) %>%
  mutate(final_sip = ifelse(final_sip == 'NANA', NA, final_sip)) %>%
  mutate(final_bus = ifelse(final_bus == 'NANA', NA, final_bus)) %>%
  mutate(county_fips = str_pad(as.character(fips), 5, "left", pad = '0')) %>%
  mutate(state_fips = substr(county_fips, 1 ,2)) %>%
  select(state_fips, county_fips, county = county_name, state = stateabbrv,
         final_sip, final_bus, pop) -> event_study_policies 


raw_policy_data %>%
  filter(is.na(county_fips)) %>%
  select(state_fips, state_source_sip = source_sip, 
         state_source_business = source_business) -> state_policy_sources

event_study_policies %>%
  left_join(raw_policy_data %>% 
              select(state_fips, county_fips, effective_date_sip, source_sip,
                     effective_date_business, source_business),
            by = c('state_fips', 'county_fips')) %>%
  left_join(state_policy_sources, by = 'state_fips') %>%
  mutate(sip_match = (final_sip == effective_date_sip)) %>%
  mutate(bus_match = (final_bus == effective_date_business)) %>% 
  mutate(state_inherited_sip = (!sip_match |
                                  (!is.na(final_sip) & 
                                     is.na(effective_date_sip)))) %>%
  mutate(state_inherited_bus = (!bus_match |
                                  (!is.na(final_bus) & 
                                     is.na(effective_date_business)))) %>%
  mutate(final_sip_source = ifelse(state_inherited_sip, 
                                   state_source_sip, 
                                   source_sip)) %>%
  mutate(final_bus_source = ifelse(state_inherited_bus, 
                                   state_source_business, 
                                   source_business)) -> compare_policies 

compare_policies %>%
  filter(!is.na(final_sip)) %>%
  summarize(state_inherited_unweighted = 
              sum(as.numeric(state_inherited_sip))/n(),
            state_inherited_weighted = 
              sum(as.numeric(state_inherited_sip)*pop)/sum(pop)) %>%
  gather(key = source, value =  share) -> state_inherit_sip_stats

compare_policies %>%
  filter(!is.na(final_bus)) %>%
  summarize(state_inherited_unweighted = 
              sum(as.numeric(state_inherited_bus))/n(),
            state_inherited_weighted = 
              sum(as.numeric(state_inherited_bus)*pop)/sum(pop)) %>%
  gather(key = source, value =  share) -> state_inherit_bus_stats

sip_shares <- compare_policies %>%
  filter(!is.na(final_sip)) %>%
  group_by(source = final_sip_source) %>%
  summarize(sip_share_unweighted = n()/nrow(compare_policies %>% filter(!is.na(final_sip))),
            sip_share_weighted = sum(pop)/sum(compare_policies %>% filter(!is.na(final_sip)) %>% pull(pop)))

sip_shares %>%
  gather(key = weighting, value = share, -source) -> gathered_sip

bus_shares <- compare_policies %>%
  filter(!is.na(final_bus)) %>%
  group_by(source = final_bus_source) %>%
  summarize(bus_share_unweighted = n()/nrow(compare_policies %>% filter(!is.na(final_bus))),
            bus_share_weighted = sum(pop)/sum(compare_policies %>% filter(!is.na(final_bus)) %>% pull(pop))) 

bus_shares %>%
  gather(key = weighting, value = share, -source) -> gathered_bus

state_shares <- bind_rows(state_inherit_sip_stats, state_inherit_bus_stats)

policy_shares <- data.frame(state_inherited = state_shares$share,
                            NYT = c(gathered_sip %>% filter(source == 'nyt') %>% pull(share), 0, 0),
                            Key = c(gathered_sip %>% filter(source == 'keystone_sip') %>% pull(share),
                                    gathered_bus %>% filter(source == 'keystone_cpv') %>% pull(share)),
                            Crw = c(gathered_sip %>% filter(source == 'crwd_lockdown') %>% pull(share),
                                    gathered_bus %>% filter(source == 'crwd_business') %>% pull(share)),
                            Hik = c(gathered_sip %>% filter(source == 'hikma_shelter') %>% pull(share),
                                    gathered_bus %>% filter(source == 'hikma_work') %>% pull(share)),
                            Man = c(gathered_sip %>% filter(source == 'manual') %>% pull(share), 0,0),
                            sip = c(0,0, gathered_bus %>% filter(source == 'effective_date_sip') %>% pull(share)))
policy_shares <- t(policy_shares)

write_gslab_table(policy_shares, "output/policy_source_table.txt", "<tab:policy_source_table>")
