*ssc install gtools

/*******************************************************************************
* Construct County-Level Policy Dataset for All Counties
* (to deal with missing in county_daily_data.csv
*******************************************************************************/
clear all
import delimited using "input/state_policies.csv", clear
tempfile state_policies
save `state_policies'

import delimited using "input/county_policies.csv", clear
ren county_fips fips
tempfile county_policies
save `county_policies'

use STATE COUNTY using "input/county_db.dta", clear
destring STATE, gen(state_fips)
destring COUNTY, gen(county)
gen fips = 1e3*state_fips + county
merge m:1 state_fips using `state_policies', nogen keep(1 3)
merge 1:1 fips using `county_policies', nogen keep(1 3)
replace effective_date_sip_state = "NA" if effective_date_sip_state==""
replace effective_date_sip_county = "NA" if effective_date_sip_county==""
keep fips effective_date_sip_state effective_date_sip_county 
tempfile counties
save `counties'

/*******************************************************************************
* Clean Data
*******************************************************************************/
use fips date visits deaths cases r_share using "input/event_study_input_data.dta", clear
merge m:1 fips using `counties', nogen assert(2 3)  //Merges on policies for missing counties, asserts that existing policies match

local pre_week_start = td(29jan2020)
local post_week_start = td(8apr2020)
gen visits_pre  = visits if inrange(date, `pre_week_start',  `pre_week_start' +6)
gen visits_post = visits if inrange(date, `post_week_start', `post_week_start'+6)

local last_day = td(2may2020)

replace deaths = . if date!=`last_day'
replace cases  = . if date!=`last_day'

replace effective_date_sip_county = "" if effective_date_sip_county=="NA"
replace effective_date_sip_state = "" if effective_date_sip_state=="NA"
ren effective_date_sip_county effective_date_sip_county_str
ren effective_date_sip_state effective_date_sip_state_str
gen effective_date_sip_county = date(effective_date_sip_county_str, "YMD")
gen effective_date_sip_state = date(effective_date_sip_state_str, "YMD")
gen sip_effective_date = min(effective_date_sip_county, effective_date_sip_state)
format effective_date_sip_county effective_date_sip_state sip_effective_date %tdDDmon

gen sip_county_preemption = effective_date_sip_state - sip_effective_date
replace sip_county_preemption = 99 if missing(effective_date_sip_state) & !missing(effective_date_sip_county)
replace sip_county_preemption = . if sip_county_preemption==0

gcollapse (firstnm) deaths cases r_share sip_effective_date sip_county_preemption ///
	(sum) visits_pre visits_post, by(fips)

gen rep_pct = 100 * r_share

gen visits_pct_ch   = 100*(visits_post-visits_pre)/visits_pre
*_pctile visits_pre, percentiles(10)
*replace visits_pct_ch = . if visits_pre<=r(r1)

merge 1:1 fips using "input/county_db", nogen assert(2 3) keepusing(id)

gen STATEFP = substr(string(fips, "%05.0f"), 1, 2)
drop if inlist(STATEFP, "60", "66", "69", "72", "78")

/*******************************************************************************
* Produce maps
*******************************************************************************/
format rep_pct visits_pct_ch %12.0f

local nq = 5

local osize = "none"
local osizes = ""
forval i=1/`nq' {
	local osizes = "`osizes' `osize'"
}

tempname state_coord

local spmap_options = `"id(id) osize(`osizes') ndsize(`osize') polygon(data("`state_coord'.dta") osize(medium)) legend(size(4))"'

foreach restr in `"if !inlist(STATEFP,"02","15","72","78")"' { //Still need to remove some pacific islands from no restriction version
	preserve
	use "input/state_coord", clear
	gen n = _n
	gen state_id = _ID
	merge m:1 state_id using "input/state_db", nogen assert(3) keepusing(STATEFP)
	drop if inlist(STATEFP, "60", "66", "69", "72", "78")
	if `"`restr'"'!="" keep `restr'
	save "`state_coord'.dta", replace
	restore
	
	local suff = cond(`"`restr'"'=="","_all", ///
		cond(`"`restr'"'==`"if STATEFP=="06""', "_california", ///
			"_continental"))
			
	foreach var of varlist sip_effective_date sip_county_preemption visits_pct_ch /*deaths cases r_share*/ {
		if inlist("`var'", "cases", "deaths") {
			local fcolor = "Oranges"
			local breaks = cond("`var'"=="cases", "0 0.5 5 50 500", "0 0.5 1.5 5 10")
			*_pctile `var', percentiles(80 95 99)
			*forval p=1/`=`nq'-2' {
			*	local breaks = "`breaks' `r(r`p')'"
			*}
			summ `var'
			local breaks = "`breaks' `r(max)'"
			local cl = "clmethod(custom) clbreaks(`breaks')"
		} 
		else {
			local fcolor = cond("`var'"=="sip_county_preemption", "Blues2", "BuYlRd")
			local cl = "clmethod(quantile) clnumber(`nq')"
		}
		
		local ndfcolor = cond("`var'"=="sip_effective_date", "red", "gs12")
		local ndlabel = cond("`var'"=="sip_effective_date", "ndlabel(No identified order)", ///
			cond("`var'"=="sip_county_preemption", "ndlabel(No state/local difference)",   ///
			""))
		local legorder = cond("`var'"=="sip_effective_date", "legorder(lohi)", "")

		spmap `var' using "input/county_coord" `restr', `spmap_options' ///
			 fcolor(`fcolor') `cl' ndfcolor(`ndfcolor') `legorder' `ndlabel'
		graph export "output/map_county`suff'_`var'.pdf", replace
	}
}

rm "`state_coord'.dta"


