# get_outcome_decomposition takes in the event study data, event study specifation 
# name and the time period to get the treatment effect and then produces
# summary statistics (initial level, total change, mandatory change, etc.) 
# for the decomposition of that particular specification's outcome variable 
# extrapolate_pretrend - specifies if the treatment effect should be linearly
# extrapollated from the trend of treatment effects pre-treatment
get_outcome_decomposition <- function(event_study_data, specification, effect_time,
                                      extrapolate_pretrend = FALSE) {
  
  # use the specficiation and effect_time to pull the 
  # outcome variable name and the treatment effect
  specification <- as.character(specification)
  outcome <- unlist(all_taus %>% filter(spec == specification) %>% distinct(outcome_variable))
  tau <- all_taus %>% filter(spec == specification, event_time == effect_time) %>% select(coef, se)
  
  # modify the treatment effect by the linear extrappolation if required
  if(extrapolate_pretrend) {
    all_taus %>%
      filter(spec == specification) %>%
      filter((event_time >= -14) & (event_time <= -1)) %>% 
      lm(as.formula("coef ~ event_time"), data =.) %>% 
      predict.lm(., se.fit = TRUE, newdata = data.frame('event_time' = c(effect_time))) -> pred
    
    tau['coef'] <- tau['coef'] - pred$fit
    tau['se'] <- sqrt(tau['se']^2 + (pred$se.fit)^2)
  }
  
  # if the outcome variable is a health outcome, 
  # filter the event_study_data to only those 
  # csa_ids that were used in the health event study
  if(str_detect(specification, 'beta')) {
    event_study_data <- event_study_data %>% filter(is_health_csa) 
  }
  
  # perform the outcome decomposition
  event_study_data %>%
    # specify the outcome variable
    mutate(outcome_variable = unlist(event_study_data[outcome])) %>%
    filter(!is.na(outcome_variable)) %>%
    group_by(csa_id, weight) %>%
    # calculate the pre, post, and t period outcomes
    summarize(pre_outcome = sum(outcome_variable*is_pre_period)/sum(is_pre_period),
              post_outcome = sum(outcome_variable*is_post_period)/sum(is_post_period),
              t_outcome = sum(outcome_variable*is_t_period)/sum(is_t_period)) %>%
    # construct summary stats comparing period outcomes
    mutate(total_change = post_outcome - pre_outcome) %>%
    mutate(pre_post_diff = pre_outcome - post_outcome) %>%
    mutate(taut_outcome =  abs(tau$coef*t_outcome)) %>%
    mutate(mandatory_change = tau$coef*t_outcome) %>%
    ungroup %>%
    # aggregate across csa_ids 
    summarize(outcome_variable = outcome, 
              unit_count = n(),
              total = weighted.mean(total_change, weight),
              total_se = sqrt(var(total_change)),
              mandatory = weighted.mean(mandatory_change, weight),
              mandatory_se = weighted.mean(t_outcome, weight)*tau$se,
              initial_level = weighted.mean(pre_outcome, weight),
              initial_se = sqrt(var(pre_outcome))) %>%
    # generate normalized total and mandatory changes by the initial level
    mutate(total_normalized = total/initial_level) %>%
    mutate(mandatory_normalized = mandatory/initial_level) %>%
    mutate(mandatory_se_normalized = mandatory_se/initial_level) %>%
    # calculate share mandatory out of the total change
    mutate(mandatory_share = mandatory_normalized/total_normalized) %>%
    mutate(mandatory_share_se = mandatory_se/abs(total_normalized*initial_level)) -> decomposition
  
  return(decomposition)
}

# run_decomposition calls on get_outcome_decomposition to produce decomposition results
# for the cartesian product of specifications and effect_times provided. 
# policy - indiciates which of sip/bus/both is being analyzed. 
# specifications - list of specs from all_taus to analyze
# effect_time - list of periods to pull treatment effects from
# is_weighted -  population weights CSAs when aggregating
# t_period_length -  specifies the length of the period 
# used to get the mean outcome just before policy implementation
# standard_R0 - indicates the assumed reproduction number in the pre_period
# restriction_period - specifies the period of policy implementations that we filter on
run_decompositions <- function(policy, specifications, effect_times, 
                               extrapolate_pretrend = FALSE,
                               is_weighted = TRUE, 
                               t_period_length = 7,
                               standard_R0 = 3,
                               restriction_period =  seq(as.Date('2020-03-01'), 
                                                         as.Date('2020-04-07'), 
                                                        by = 1)) {
  # set-up data in alignment with specified parameters
  csa_event_study %>%
    # filter to the appropriate policy 
    filter(policy_type == policy) %>%
    # filter to policies implemented within the restriction period
    filter(policy_date %in% restriction_period) %>%
    # define t_period going back t_period_length days before implmenetation
    mutate(is_t_period = as.integer((event_time >= - (t_period_length + 1)) & 
                                      (event_time < - 1))) %>%
    # filter to csa_id/days that one of pre, post, and t periods
    filter(is_pre_period + is_post_period + is_t_period > 0) %>% 
    # if results are weighted use population weights, otherwise set weights to uniform
    rowwise %>% mutate(weight = ifelse(is_weighted, pop, 1)) %>% ungroup %>%
    # define the transmission rate beta for each gamma using the pre_period
    # standard_R0 when neccesary
    mutate(beta_0 = ifelse(is.na(beta_0), standard_R0, beta_0)) %>%
    mutate(beta_12 = ifelse(is.na(beta_12), standard_R0/12, beta_12)) %>%
    mutate(beta_9 = ifelse(is.na(beta_9), standard_R0/9, beta_9)) %>%
    mutate(beta_6 = ifelse(is.na(beta_6), standard_R0/6, beta_6)) %>%
    mutate(beta_3 = ifelse(is.na(beta_3), standard_R0/3, beta_3)) -> event_study_data
  
  # create grid with cartesian product of specifications and effect_times
  data.frame(expand.grid(policy_type = c(policy), 
                         spec = specifications, 
                         effect_time = effect_times)) %>%
    group_by(policy_type, spec, effect_time)  %>%
    nest() %>%
    select(-data) %>%
    # call get_outcome_decomposition for each specification and time period
    mutate(results = map2(spec, effect_time, 
                          ~get_outcome_decomposition(event_study_data, .x, .y, 
                                                     extrapolate_pretrend))) %>%
    unnest(results) %>% ungroup -> decompositions
  
  return(decompositions)
}
  
# format_decomp_results prettifies the run_decomposition output to look
# like we want it for the lyx tables. Standard errors are formatted under
# the appropriate estimates along with empty spaces. 
# decomp - the run_decomposition output to format
# include_spec_label - indicates if we want to include a column of the specification labels
# include_totals - indicates if we want to include totals in the formatting. sometimes the totals
# will not change across decomposition results (like in the appendix table)
format_decomp_results <- function(decomp, include_spec_label = FALSE,
                                  include_totals = TRUE) {
  decomp %>%
    select(spec, total = total_normalized, 
           mandatory = mandatory_se_normalized) %>%
    rowwise %>%
    mutate(total = '.') -> se_joiner
  
  decomp %>%
    select(spec, total = total_normalized, 
           mandatory = mandatory_normalized) %>%
    mutate(total = as.character(total)) %>%
    bind_rows(se_joiner) %>%
    arrange(spec) -> cleaned
  
  if(!include_spec_label) {
    cleaned <- cleaned %>% select(-spec)
  } 
  
  if(!include_totals) {
    cleaned <- cleaned %>% select(-total)
  }
  
  return(cleaned)
}

write_gslab_table <- function(TABLE, output, header){
  write.table(TABLE, file = output, row.names = FALSE, col.names = FALSE, sep = "\t")
  table <- read_file(output) # http://stackoverflow.com/questions/9068397/import-text-file-as-single-character-string
  table <- sprintf("%s\n%s", header, gsub('"', '', table))
  write_file(table, output)
}