library(dplyr)
library(readr)
library(stringr)
library(tidyr)
library(purrr)
library(ggplot2)
library(readstata13)
source('code/decomp_helper_functions.R')

############################### DATA IMPORT ###############################

# read in policy data 
full_policy_data <- read_csv('../data/output/full_policy_data.csv')
# read in event study data 
csa_event_study_sip_raw    <- read.dta13('../data/output/csa_sip_event_study_input_data.dta')
csa_event_study_bus_raw    <- read.dta13('../data/output/csa_bus_event_study_input_data.dta')
csa_event_study_both_raw   <- read.dta13('../data/output/csa_both_event_study_input_data.dta')

# create empty data frame for all the event study result information
all_taus <- data.frame(spec = character(), event_time = integer(),
                       coef = numeric(), se = numeric(), 
                       outcome_variable = character())

# collect all the event study filenames
csa_event_study_folder <- list.files('output/csa')
csa_event_study_files  <- csa_event_study_folder[grepl('.txt', csa_event_study_folder)]

# remove summary .txt files which do not contain 
# treatment effects for visits/econ outcomes
csa_event_study_files <- csa_event_study_files[!grepl('sum.txt', csa_event_study_files)]
csa_event_study_files <- csa_event_study_files[!grepl('table', csa_event_study_files)]

# this will have to be manually updated if filename conventions change
# simply maps filenames to the associated outcome variables 
# with the outcome variable as listed in the csa_event_study data
map_file_to_outcome <- tibble(filename = csa_event_study_files) %>%
  # extract outcome variables from filenames 
  mutate(outcome_variable = sub('\\.txt.*', "", filename)) %>% 
  mutate(outcome_variable = sub('\\_bus.*', "", outcome_variable)) %>% 
  mutate(outcome_variable = sub('\\_both.*', "", outcome_variable)) %>% 
  mutate(outcome_variable = sub('\\_open.*', "", outcome_variable)) %>% 
  mutate(outcome_variable = sub('\\_bin.*', "", outcome_variable)) %>%
  mutate(outcome_variable = sub('\\_sip.*', "", outcome_variable)) %>%
  mutate(outcome_variable = sub('log_', "", outcome_variable)) %>%
  mutate(spec =  str_remove(filename, '.txt'))  

# add each visit/econ event study result 
for(result in map_file_to_outcome$filename) {
  results <- read_tsv(paste0('output/csa/', result)) 
  results %>% 
    rowwise %>% 
    mutate(filename = result)  %>%
    left_join(map_file_to_outcome, by = 'filename')  %>%
    select(-filename)  -> new_spec
  all_taus <- all_taus %>% bind_rows(new_spec)
}

# read in the contact rate treatment effects
csa_contact_rate_taus <- read_tsv('output/csa/table_agg_reg.txt', 
                                  skip = 1,
                                  col_names = FALSE) 
colnames(csa_contact_rate_taus) <- c('beta_0', 'beta_12', 'beta_9', 
                                     'beta_6', 'beta_4')

# rows 1:2 and 5:6 contain the baseline and trump/urban fe specs respectively
csa_contact_rate_taus[c(1:2, 5:6),] %>%
  cbind('temp_spec' = rep(c('baseline', 'baseline_se', 
                            'trump_urban', 'trump_urban_se'))) %>%
  gather(key = gamma, value = estimate, - temp_spec) %>%
  mutate(is_se = ifelse(str_detect(temp_spec, '_se'), 'se', 'coef')) %>%
  mutate(temp_spec = str_remove(temp_spec, '_se')) %>%
  spread(key = is_se, value = estimate) %>%
  rowwise %>%
  mutate(outcome_variable = gamma) %>%
  mutate(event_time = 1) %>%
  ungroup %>%
  unite('spec', c(temp_spec, gamma))  -> contact_rate_taus

# join the contact rate treatment effects
all_taus <- all_taus %>% bind_rows(contact_rate_taus)

######################## FORMAT DATA ##########################################

# define pre and post periods
pre_period  <- seq(as.Date('2020-02-01'), as.Date('2020-02-07'), by = 1)
post_period <- seq(as.Date('2020-04-12'), as.Date('2020-04-18'), by = 1)

# combine and clean up the event study data for all policy types
csa_event_study_sip_raw %>%
  # join all csa_event_study data indicating policy type
  mutate(policy_type = 'sip') %>%
  bind_rows(csa_event_study_bus_raw %>% mutate(policy_type = 'bus')) %>%
  bind_rows(csa_event_study_both_raw %>% mutate(policy_type = 'both')) %>% 
  # collect policy effective dates into one variable
  mutate(policy_date = ifelse(is.na(first_day_order_bus),
                              as.character(first_day_order_sip),
                              as.character(first_day_order_bus))) %>%
  # format date variables
  mutate(policy_date = as.Date(policy_date)) %>%
  mutate(date = as.Date(date)) %>% 
  # generate indicator for periods and get event_time
  mutate(is_pre_period = as.numeric(date %in% pre_period)) %>%
  mutate(is_post_period = as.numeric(date %in% post_period)) %>%
  mutate(event_time = as.numeric(date - policy_date)) %>%
  # generate CSA id from csacode and csa partition w/ order_date
  rowwise %>%
  mutate(csa_id = paste0(csacode, '_', csa_order_date)) %>%
  ungroup -> csa_event_study

# collect the csa_ids used in the health event 
# study to generate the treatment effects 
health_event_study_csa_ids <- unique(csa_event_study[csa_event_study$estimation_sample == 1, ]$csa_id)

# join the infected counts and generate betas for each choice of gamma
csa_event_study %>% 
  # join in previous day's cases
  mutate(next_date = date + 1) %>%
  left_join(csa_event_study %>% 
              select(next_date = date, csa_id, next_day_cases = cases, policy_type),
                     by = c('next_date', 'csa_id', 'policy_type')) %>%
  mutate(beta_0  = (next_day_cases - cases) / I_t_0)  %>%
  mutate(beta_12 = (next_day_cases - cases) / I_t_12) %>%
  mutate(beta_9  = (next_day_cases - cases) / I_t_9)  %>%
  mutate(beta_6  = (next_day_cases - cases) / I_t_6)  %>%
  mutate(beta_3  = (next_day_cases - cases) / I_t_3)  %>%
  # restrict to CSAs that were in the health event study samples
  mutate(is_health_csa = csa_id %in% health_event_study_csa_ids ) -> csa_event_study

##################################### RUN DECOMPOSITIONS #####################################

# list the visits/econ outcome specifications we look a
core_vars <- c('log_visits', 'log_sum_wages', 'log_count_hours_pos',  
               'log_num_transactions', 'log_total_spend')

# add the appropriate policy to the end of the visits/econ specifications
sip_specs <- unlist(map(core_vars, ~paste0(.x, '_sip')))

bus_specs <- unlist(map(core_vars, ~paste0(.x, '_bus')))

both_specs <- unlist(map(core_vars, ~paste0(.x, '_both')))

# get the primary visits/econ outcomes for sip/bus/both policies in period t + 1
decomp_1 <- run_decompositions(policy = 'sip',
                               specifications = sip_specs,
                               effect_times = c(1)) 

decomp_2 <- run_decompositions(policy = 'bus',
                               specifications = bus_specs,
                               effect_times = c(1)) 

decomp_3 <- run_decompositions(policy = 'both',
                               specifications = both_specs,
                               effect_times = c(1)) 

# prettify the results for the paper table
format_decomp_results(decomp_1) %>%
  bind_cols(format_decomp_results(decomp_2)) %>%
  bind_cols(format_decomp_results(decomp_3)) %>%
  rowwise -> sd_econ_results

# list health outcome specifications we care about
health_vars <- c('baseline_beta_12', 'baseline_beta_9', 'baseline_beta_6')

# get health decomposition with sip policy and R0 = 2.7
health_27_sip <- run_decompositions('sip', 
                                    health_vars,
                                    c(1),
                                    standard_R0 = 2.7) 

# prettify health outcomes for paper table 
health_27_sip %>%
  filter(str_detect(spec, '12')) %>%
  format_decomp_results() %>%
  bind_cols(health_27_sip %>%
              filter(str_detect(spec, '9')) %>%
              format_decomp_results()) %>%
  bind_cols(health_27_sip %>%
              filter(str_detect(spec, '6')) %>%
              format_decomp_results())-> health_27

# get health decomposition with sip policy and R0 = 2.7
health_3_sip <- run_decompositions('sip', 
                                    health_vars,
                                    c(1),
                                    standard_R0 = 3.0) 

# prettify health outcomes for paper table 
health_3_sip %>%
  filter(str_detect(spec, '12')) %>%
  format_decomp_results() %>%
  bind_cols(health_3_sip %>%
              filter(str_detect(spec, '9')) %>%
              format_decomp_results()) %>%
  bind_cols(health_3_sip %>%
              filter(str_detect(spec, '6')) %>%
              format_decomp_results())-> health_3

# get health decomposition with sip policy and R0 = 2.7
health_33_sip <- run_decompositions('sip', 
                                   health_vars,
                                   c(1),
                                   standard_R0 = 3.3) 

# prettify health outcomes for paper table 
health_33_sip %>%
  filter(str_detect(spec, '12')) %>%
  format_decomp_results() %>%
  bind_cols(health_33_sip %>%
              filter(str_detect(spec, '9')) %>%
              format_decomp_results()) %>%
  bind_cols(health_33_sip %>%
              filter(str_detect(spec, '6')) %>%
              format_decomp_results())-> health_33

# join health decomposition results
health_decomp <- bind_rows(health_27, health_3, health_33)

# join visits/econ/health decomposition into table in the appropriate
# format for the paper. 
decomp_table <- rbind(as.matrix(sd_econ_results), as.matrix(health_decomp))

write_gslab_table(decomp_table, "output/decomp/table_decomp.txt", 
                  "<tab:table_decomp>")

d_table <- data.frame(decomp_table[1:10,c(1,2)])
colnames(d_table) <- c('total', 'policy')
d_table$pct <- (as.numeric(d_table$policy)/as.numeric(d_table$total)) * 100
d_table$pct <- ifelse(is.na(d_table$pct), '.', as.character(d_table$pct))

write_gslab_table(d_table, "output/decomp/table_decomp_pct.txt",
                  "<tab:table_decomp_pct>")

c_table <- data.frame(decomp_table[11:16,c(5,6)])
colnames(c_table) <- c('total', 'policy')
c_table$pct <- (as.numeric(c_table$policy)/as.numeric(c_table$total)) * 100
c_table$pct <- ifelse(is.na(c_table$pct), '.', as.character(c_table$pct))

write_gslab_table(c_table, "output/decomp/table_contact_pct.txt",
                  "<tab:table_contact_pct>")

# get mandatory/total shares for each visits/econ/health outcome
# variable to be used in the decomposition plots
decomp_1 %>%
  bind_rows(decomp_2) %>%
  bind_rows(decomp_3) %>%
  bind_rows(health_3_sip %>% mutate(R0 = '3')) %>%
  bind_rows(health_27_sip %>% mutate(R0 = '2.7')) %>% 
  bind_rows(health_33_sip %>% mutate(R0 = '3.3')) %>% 
  select(spec, mandatory_share, mandatory_share_se, R0) %>%
  filter(!str_detect(spec,'log_total_spend')) %>%
  filter(!str_detect(spec,'log_num_transactions')) %>% 
  mutate(outcome = ifelse(str_detect(spec, 'log_visits'), 'Visits', 
                          ifelse(str_detect(spec, 'sum_wages'), 'Wages',
                                 ifelse(str_detect(spec, 'count_hours'), 'Employment',
                                        'Contact Rate')))) %>%
  mutate(pretty_spec = ifelse(str_detect(spec, 'both'), 'Both',
                              ifelse(str_detect(spec, 'bus'), 'Business', 'SIP'))) %>%
  mutate(gamma = ifelse(str_detect(spec, '6'), '1/6',
                        ifelse(str_detect(spec, '9'), '1/9', 
                               ifelse(str_detect(spec, '12'), '1/12',
                                      NA)))) -> data_for_decomp_plot

write_csv(data_for_decomp_plot, 'output/decomp/data_for_decomp_plot.csv')


################## APPENDIX ##########################

# get the primary visits/econ outcomes for sip policies 
# in period t + 1, t + 20, and 
# t + 20 with the treatment effect linearly extrappolated 
sip_1 <- run_decompositions('sip',
                            sip_specs,  
                            c(1)) 
 
sip_2 <- run_decompositions('sip',
                            sip_specs, 
                            c(20)) 


sip_3 <- run_decompositions('sip',
                            sip_specs, 
                            c(20),
                            extrapolate_pretrend = TRUE) 

#prettify sip appendix results
format_decomp_results(sip_1) %>%
  # since the total change is constant across specs, only include total change in first decomp
  bind_cols(format_decomp_results(sip_2, include_totals = FALSE)) %>%
  bind_cols(format_decomp_results(sip_3, include_totals = FALSE)) %>%
  rowwise  -> sip_app_results

# get the primary visits/econ outcomes for bus policies 
# in period t + 1, t + 20, and 
# t + 20 with the treatment effect linearly extrappolated
bus_1 <- run_decompositions('bus',
                            bus_specs,  
                            c(1)) 

bus_2 <- run_decompositions('bus',
                            bus_specs, 
                            c(20)) 


bus_3 <- run_decompositions('bus',
                            bus_specs, 
                            c(20),
                            extrapolate_pretrend = TRUE) 

# prettify bus appendix results
format_decomp_results(bus_1) %>%
  # since the total change is constant across specs, only include total change in first decomp
  bind_cols(format_decomp_results(bus_2, include_totals = FALSE)) %>%
  bind_cols(format_decomp_results(bus_3, include_totals = FALSE)) %>%
  rowwise  -> bus_app_results

# combine bus and sup results
app_results <- bind_rows(sip_app_results, bus_app_results)

write_gslab_table(app_results, "output/decomp/table_decomp_appendix.txt", 
                  "<tab:table_decomp_appendix>")
