/***
Purpose: Run preliminary first stage of social distancing on shelter in place orders
Created: BG 2020-4-3
***/

clear all
set maxvar 20000
adopath + "../lib/ado"
set scheme bg_5by8

*-------------------------------------------------------------------------------------------------
* Event study parameters
*-------------------------------------------------------------------------------------------------

* Default number of leads 
global leads=21

* Lags 
global lags=21

* Default clustering variable 
global def_clust "csacode"

* Additional graph options
global options1 "ylabel(-0.4(0.2)0.4) xlabel(-21(7)21) xmtick(##7) legend(off)"
global options2 "ylabel(-1.2(0.4)1.2) xlabel(-21(7)21) xmtick(##7) legend(off)"
global options3 "ylabel(-0.3(0.1)0.3) yscale(range(-.35 .35)) xlabel(-21(7)21) xmtick(##7) legend(off)"

* Graph type
global figext pdf

*------------------------------------------------------------------------------------------------
* Run various event study specifications
*-------------------------------------------------------------------------------------------------

* All very sensitive to county vs. treatment FE because of selection by date of first order
* Default spec has logs, CSA clustering, balanced sample, and population weights
* Only make labels and titles for deviation from default spec

* Run event studies in a massive loop that goes over outcomes, SIP/bus/both
foreach p in sip bus both open {
	local xtitle = cond("`p'"=="open", `"xtitle("Days Relative to Order Removal")"', "")
	
	* Read in data
	use "input/csa_`p'_event_study_input_data", clear
	if "`p'" != "open" {
		keep if date >= mdy(2, 1, 2020) & date < mdy(5, 1, 2020)
	}
	if "`p'" == "open" {
		keep if date >= mdy(3, 1, 2020) & date < mdy(8, 1, 2020)
	}
	assert ~mi(csacode)

	foreach y in log_visits log_sum_hours log_total_spend log_num_transactions log_sum_wages log_count_hours_pos {
		
		* Set graph options
		global opt ${options1}
		if ( "`y'"=="log_sum_wages" | "`y'"=="log_count_hours_pos" | "`y'"=="log_sum_hours") & "`p'"=="both" {
			global opt ${options2}
		}
		
		* Default spec
		eventplot `y' lead* lag* [aw=pop], ///
			absorb(csa_order_date date) treatvar(treatment) eventtime(event_day) ///
			leads(${leads}) lags(${lags}) omit(1) clustervar(${def_clust}) ${opt} ///
			output("output/csa/`y'_`p'") `xtitle'
	}
}
 
*-------------------------------------------------------------------------------------------------
* Baseline event studies at the county level 
*-------------------------------------------------------------------------------------------------

* Switch to state level clustering
global def_clust "state_fips"

* Go back to county level data
use "input/event_study_input_data", clear
keep if date >= mdy(2, 1, 2020) & date < mdy(5, 1, 2020)

* Baseline outcomes at the county level
foreach y in log_visits log_sum_hours log_total_spend log_num_transactions log_sum_wages log_count_hours_pos {
	* Default spec
	eventplot `y' lead* lag* [aw=pop], ///
		absorb(fips date) treatvar(treatment) eventtime(event_day) ///
		leads(${leads}) lags(${lags}) omit(1) clustervar(${def_clust}) ${options3} ///
		output("output/county/`y'_sip_cty")
}
