/***
Purpose: Investigate treatment effect assumptions
Created: BG 2020-6-21
***/

clear all
set maxvar 20000
adopath + "../lib/ado"
set scheme bg_5by8

*-------------------------------------------------------------------------------------------------
* Run separate event studies by order date to study heterogeneity in the treatment effects
*-------------------------------------------------------------------------------------------------

use "input/csa_sip_event_study_input_data", clear
keep if date >= mdy(2, 1, 2020) & date < mdy(5, 1, 2020)

* Assign a type of "treatment" and "control"
g treat=inrange(first_day_order_sip, td(15mar2020), td(5apr2020))
g control=first_day_order_sip>td(6apr2020)
keep if treat==1 | control==1

* Make empty variables to store results
su first_day_order_sip if treat==1
local fday `r(min)'
local lday `r(max)'
g sip_date=_n+`fday'-1 if _n<=(`lday'-`fday'+1)
format sip_date %tdMon_dd
g coef_day_after=.

* Loop over order dates to store results
quietly forvalues d=`fday'/`lday' {
	noi di `d'
	eventplot log_visits lead* lag* [aw=pop] if first_day_order_sip==`d' | control==1, ///
		absorb(csa_order_date date) treatvar(treatment) eventtime(event_day) ///
		leads(21) lags(21) omit(1) clustervar(csacode) storecoef
		
	su coef if event_time==1
	assert `r(N)'==1
	replace coef_day_after=`r(mean)' if sip_date==`d'
	drop event_time se coef top bot
}

* Plot event study estimates
reg coef_day_after sip_date, robust
local b  : di %05.4f _b[sip_date]
local se : di %05.4f _se[sip_date]
twoway ///
	scatter coef_day_after sip_date || ///
	lfit coef_day_after sip_date, lcolor("scheme p1") ///
	xtitle("Date of Stay-at-Home Order") xlabel(`fday'(5)`lday') xscale(range(`fday' `=`lday'+1')) ///
	ytitle("Estimate of {&omega}{subscript:1}") ylabel(-.4(.1).1, format(%02.1f) gmax) ///
	text(0.05 `=`lday'-1' "Slope: `b' " "(`se')", place(w) just(right)) ///
	legend(off)
graph export "output/csa/log_visits_te_by_order_date.pdf", replace
