clear all
set maxvar 20000
adopath + "../lib/ado"
set scheme bg_5by8
set seed 2020

* Number of folds
local folds 10
local seed 2020

local min_val = 1e-6

do input/lib/stata/write_latex

*-------------------------------------------------------------------------------------------------
* Compute FEs 
*-------------------------------------------------------------------------------------------------

* Load and preliminary cleaning
use "input/csa_sip_event_study_input_data.dta", clear

* Get Event Study Estimates for Input
keep if date >= mdy(2, 1, 2020) & date < mdy(5, 1, 2020)
global leads=21
global lags=21
global def_clust "csacode"
global options1 "ylabel(-0.4(0.2)0.4) xlabel(-21(7)21) xmtick(##7) legend(off)"
eventplot log_visits lead* lag* [aw=pop], absorb(csa_order_date date) treatvar(treatment) eventtime(event_day) leads(${leads}) lags(${lags}) omit(1) clustervar(${def_clust}) ${options1} output("output/poi_visits_for_input") `xtitle'
local poi_effect = _b[lag_1]

* Clean
local unit csa_order_date
xtset `unit' event_day
keep if estimation_sample == 1

gen     treat_on = 0
replace treat_on = 1 if event_day >= 0 & event_day != .

bysort `unit': egen total_tests = sum(new_tests)

* Estimate impact on health outcomes
reghdfe log_new_cases pre_event_window event_window post post_post log_I_t_6 [aw=pop] if estimation_sample == 1, absorb(date) cluster(csacode)

local treatment_6 = _b[post]
gen beta_t_6      = (f_cases - cases) / I_t_6 
replace beta_t_6  = `min_val' if beta_t_6 < `min_val'
gen log_beta_t_6  = log(beta_t_6)

* Get FEs and do descriptives
local Lambda_0         = (`treatment_6' / `poi_effect')
gen log_beta_t_6_const = log_beta_t_6 - `Lambda_0' * delta_log_visits

xtset `unit' date
xtreg log_beta_t_6_const i.date if date >= mdy(3,15,2020) & date < mdy(5,1,2020), fe vce(cluster csacode)
predict temp, u
bysort `unit': egen fe = max(temp)
drop temp
predict date_val_add, xb

* Produce the raw fixed effects
xtreg log_beta_t_6 i.date if date >= mdy(3,15,2020) & date < mdy(5,1,2020), fe vce(cluster csacode)
predict temp, u
bysort `unit': egen fe_raw = max(temp)
drop temp


*-------------------------------------------------------------------------------------------------
* Compute Counterfactuals
*-------------------------------------------------------------------------------------------------
run code/functions
compute_shares

* SF Distancing Counterfactual
cap drop temp
gen temp = delta_log_visits if `unit' == 263 
bysort date: egen sf_delta_log_visits = mode(temp)
drop temp
xtset `unit' date
gen beta_sf_distancing = max(exp( log_beta_t_6 - `Lambda_0' * (delta_log_visits - sf_delta_log_visits) ), `min_val')
compute_cases, suffix(_sf_distancing) gam(.1666667) beta(beta_sf_distancing)

* March 17 Policy Counterfactual (Proportional)
gen 	log_visits_policy_prop  = log_visits
replace log_visits_policy_prop  = log_visits + `poi_effect'   if date >= mdy(3, 17, 2020)
replace log_visits_policy_prop  = log_visits          if treat_on == 1
gen 	delta_log_visits_policy_prop = log_visits_policy_prop - base_log_visits
gen     beta_policy_prop = max(exp( log_beta_t_6 - `Lambda_0' * (delta_log_visits - delta_log_visits_policy_prop) ), `min_val')
compute_cases, suffix(_policy_prop) gam(.1666667) beta(beta_policy_prop)

* March 17 Policy Counterfactual (Fixed Level)
gen 	log_visits_policy_fix   = log_visits
replace log_visits_policy_fix   = log(base_visits * .35)   if date >= mdy(3, 17, 2020)
gen 	delta_log_visits_policy_fix = log_visits_policy_fix - base_log_visits
gen     beta_policy_fix  = max(exp( log_beta_t_6 - `Lambda_0' * (delta_log_visits - delta_log_visits_policy_fix) ), `min_val')
compute_cases, suffix(_policy_fix) gam(.1666667) beta(beta_policy_fix)

* No Policy Counterfactual (Proportional)
gen 	log_visits_no_policy  = log_visits
replace log_visits_no_policy  = log_visits - `poi_effect'   if treat_on == 1
gen 	delta_log_visits_no_policy = log_visits_no_policy - base_log_visits
gen     beta_no_policy = max(exp( log_beta_t_6 - `Lambda_0' * (delta_log_visits - delta_log_visits_no_policy) ), `min_val')
compute_cases, suffix(_no_policy) gam(.1666667) beta(beta_no_policy)

* Test Compute Cases
gen 	log_visits_test  = log_visits
gen 	delta_log_visits_test = log_visits_test - base_log_visits
gen     beta_test = max(exp( log_beta_t_6 - `Lambda_0' * (delta_log_visits - delta_log_visits_test) ), `min_val')
compute_cases, suffix(_test) gam(.1666667) beta(beta_test)


 * Aggregate counterfactual
 preserve
 collapse (sum) cases*, by(date)
 
 replace cases               = cases               / 1000000
 replace cases_test          = cases_test          / 1000000
 replace cases_policy_prop   = cases_policy_prop   / 1000000
 replace cases_no_policy     = cases_no_policy     / 1000000
 replace cases_policy_fix    = cases_policy_fix    / 1000000
 replace cases_sf_distancing = cases_sf_distancing / 1000000

 keep if date < mdy(5, 1, 2020) & date >= mdy(3, 1, 2020)
 twoway (line cases_no_policy date, mcolor("scheme p1") lpattern("dash")) 		///
	(line cases date, mcolor("scheme p2") lpattern("solid") lwidth(*1.5)) 		///
	(line cases_policy_prop date, mcolor("scheme p3") lpattern("dash_dot"))		///
	(line cases_sf_distancing date, mcolor("scheme p4") lpattern("shortdash"))	///
	(line cases_policy_fix date, mcolor("scheme p5") lpattern("longdash"))		///
	, legend(label(1 "No Policy - Proportional") label(2 "Observed") label(3 "March 17 Policy - Proportional") label(4 "SF Distancing") label(5 "March 17 Policy - Fixed") ) xtitle("") ytitle("Cases (millions)")  xtitle("") xlabel(`=td(01mar2020)'(14)`=td(01may2020)', format(%tdMon_dd)) xmtick(##2)
 graph export "output/counterfactual_cases.pdf", replace
 

 sum cases_no_policy if date == mdy(4, 30, 2020)
 latex_rounded, name(casesnopolicy)  value(1000 * `r(mean)') digits(1)
 latex_rounded, name(deathsnopolicy) value(1000 * `r(mean)' * 0.03) digits(1)

 sum cases if date == mdy(4, 30, 2020)
 latex_rounded, name(casesactual)  value(1000 * `r(mean)') digits(1)
 latex_rounded, name(deathsactual) value(1000 * `r(mean)' * 0.03) digits(1)

 sum cases_sf_distancing if date == mdy(4, 30, 2020)
 latex_rounded, name(casessfamount)  value(1000 * `r(mean)') digits(1)
 latex_rounded, name(deathssfamount) value(1000 * `r(mean)' * 0.03) digits(1)

 sum cases_policy_prop if date == mdy(4, 30, 2020)
 latex_rounded, name(casessfdate)  value(1000 * `r(mean)') digits(1)
 latex_rounded, name(deathssfdate) value(1000 * `r(mean)' * 0.03) digits(1)

  sum cases_policy_fix if date == mdy(4, 30, 2020)
 latex_rounded, name(casessfdatefix)  value(1000 * `r(mean)') digits(1)
 latex_rounded, name(deathssfdatefix) value(1000 * `r(mean)' * 0.03) digits(1)

 restore

*-------------------------------------------------------------------------------------------------
* Find Best Lasso Model AND additive decomposition
*-------------------------------------------------------------------------------------------------

* Convert pop vars to logs
foreach x of varlist pop pop_density total_tests {
	g log_`x' = log(`x')
}

* Add 1 to the log of international flights to deal with 0s
rename inbound_international_flights inter_flights
gen log_inter_flights=log(inter_flights+1)

* Run lasso model to predict fixed effects
global folds 10
global seed 2020
local lasso_vars log_pop_density log_pop sh_pop_urban summer_tmmx winter_tmmx sh_pop_commute_auto log_inter_flights r_share sh_pop_age_65plus sh_pop_age_under18 sh_pop_race_black sh_pop_race_white sh_pop_race_asian sh_pop_race_other sh_pop_hispanic sh_pop_educ_bachelors_plu
lasso linear fe `lasso_vars' if date == mdy(4,30,2020), folds(${folds}) grid(500, ratio(1e-10)) rseed(${seed})

* Assign a value add = to b_lasso*covariate for each set of variables
lassocoef
create_value_add, unit(`unit')

* Output a sheet of the value add of different covariates
preserve 
g distancing_val_add = `Lambda_0' * delta_log_visits
g policy_val_add = `Lambda_0' * (`poi_effect' * treat_on)
keep if date >=  mdy(3, 15, 2020) & date <  mdy(5, 1, 2020)
outsheet csa_order_date date pop log_beta_t_6 distancing_val_add policy_val_add date_val_add lasso_pred_fe density_val_add climate_val_add transport_val_add race_val_add party_val_add education_val_add demographics_val_add using "output/csa_value_add_means.txt", replace
restore

preserve 
g log_cases_per_pop       = log((cases / pop) * 100000)
g log_deaths_per_pop      = log((deaths / pop) * 100000)
replace summer_tmmx = (summer_tmmx - 273.15) * 9/5 + 32
replace winter_tmmx = (winter_tmmx - 273.15) * 9/5 + 32
keep if date >=  mdy(3, 15, 2020) & date <  mdy(5, 1, 2020)
collapse (mean) log_beta_t_6 log_visits delta_log_visits treat_on `lasso_vars' (max) log_cases_per_pop log_deaths_per_pop, by(csa_order_date)
outsheet csa_order_date log_beta_t_6 log_cases_per_pop log_deaths_per_pop log_visits delta_log_visits treat_on `lasso_vars' using "output/csa_covariate_means.csv", replace comma
restore

* Produce additive decomposition table
global Lambda_0 = `Lambda_0'
global poi_effect = `poi_effect'
make_add_decomp_tables, unit(`unit')
get_scatter_data, unit(`unit')

*-------------------------------------------------------------------------------------------------
* Build coefficient plots of relationship to fixed effects
*-------------------------------------------------------------------------------------------------
* Keep a single row of data per CSA code
keep if date == mdy(4,30,2020)

* List of variables to include in the plots
local plotvars ///
	log_total_tests summer_tmmx winter_tmmx sh_pop_commute_auto log_inter_flights r_share sh_pop_age_65plus sh_pop_age_under18 sh_pop_race_black sh_pop_race_white sh_pop_race_asian sh_pop_race_other sh_pop_hispanic  sh_pop_educ_bachelors_plu sh_pop_urban log_pop_density log_pop
	
* Count number of variables 
local x_count : word count `plotvars'

* Standardize all the variables
foreach x in fe fe_raw `plotvars' {
	quietly su `x'
	g z_`x'=(`x'-`r(mean)')/`r(sd)'
}

* Variables to store results
g var_name=""
g var_id=.

* Loop over the raw and visit residualized specifications
foreach y in fe fe_raw {
	
	* Results
	g b_`y'=.
	g se_`y'=.
	g b_`y'_pop_control=.
	g se_`y'_pop_control=.
	local j=1

	foreach x of local plotvars {
		
		* Fill in var_name
		replace var_name="`x'" in `j'
		replace var_id=`j' in `j'
		
		* Univariate regression
		quietly reg z_`y' z_`x', robust
		replace b_`y'=_b[z_`x'] in `j'
		replace se_`y'=_se[z_`x'] in `j'
		
		* Population control regression
		quietly reg z_`y' z_log_pop z_`x', robust
		if "`x'"!="log_pop" {
			replace b_`y'_pop_control=_b[z_`x'] in `j'
			replace se_`y'_pop_control=_se[z_`x'] in `j'
		}
		
		* Index up loop
		local j=`j'+1	
	}
	
	* Generate standard error bars
	g top_`y'=b_`y'+1.96*se_`y'
	g bot_`y'=b_`y'-1.96*se_`y'
	g top_`y'_pop_control=b_`y'_pop_control+1.96*se_`y'_pop_control
	g bot_`y'_pop_control=b_`y'_pop_control-1.96*se_`y'_pop_control
}

keep var_name var_id b_fe* se_fe* top_fe* bot_fe*

* Add 1 for a space between pop vars
su var_id if var_name=="sh_pop_urban"
replace var_id=var_id+0.5 if var_id>=`r(mean)'

* For labels
list var_id var_name if ~mi(var_id)

* Make a version of the plot for both sets of results
foreach y in fe fe_raw {

	* Univarite regressions
	twoway ///
		dot b_`y' var_id, dstyle(none) horizontal mcolor("scheme p1") msize(*.7) || ///
		rcap top_`y' bot_`y' var_id, horizontal lcolor("scheme p1") lwidth(*.7) ///
		xtitle("Standardized Coefficient") ///
		xlabel(-0.75(.25)0.75) xline(0, lpattern(dash) lcolor(black)) ///
		ytitle("") ylabel( ///
			1 "Log Tests" ///
			2 "Avg. Winter Temp." ///
			3 "Avg. Summer Temp." ///
			4 "Share Commute by Car" ///
			5 "Log International Flights" ///
			6 "Republican Vote Share" ///
			7 "Share Age {&ge}65" ///
			8 "Share Age <18" ///
			9 "Share Race - Black" ///
			10 "Share Race - White" ///
			11 "Share Race - Asian" ///
			12 "Share Race - Other" ///
			13 "Share Hispanic" ///
			14 "Share 4-year Degree" ///
			15.5 "Frac. Urban" ///
			16.5 "Log Pop. Density" ///
			17.5 "Log Pop." ///
			, labsize(*0.8)) ///
		legend(off)
	graph export "output/`y'_coefficient_plot.pdf", replace
	
	* Population Control
	twoway ///
		dot b_`y'_pop_control var_id, dstyle(none) horizontal mcolor("scheme p1") msize(*.7) || ///
		rcap top_`y'_pop_control bot_`y'_pop_control var_id, horizontal lcolor("scheme p1") lwidth(*.7) ///
		xtitle("Standardized Coefficient") ///
		xlabel(-0.75(.25)0.75) xline(0, lpattern(dash) lcolor(black)) ///
		ytitle("") ylabel( ///
			1 "Log Tests" ///
			2 "Avg. Winter Temp." ///
			3 "Avg. Summer Temp." ///
			4 "Share Commute by Car" ///
			5 "Log International Flights" ///
			6 "Republican Vote Share" ///
			7 "Share Age {&ge}65" ///
			8 "Share Age <18" ///
			9 "Share Race - Black" ///
			10 "Share Race - White" ///
			11 "Share Race - Asian" ///
			12 "Share Race - Other" ///
			13 "Share Hispanic" ///
			14 "Share 4-year Degree" ///
			15.5 "Frac. Urban" ///
			16.5 "Log Pop. Density" ///
			17.5 "Log Pop." ///
			, labsize(*0.8)) ///
		legend(off)
	graph export "output/`y'_pop_cntrl_coefficient_plot.pdf", replace
}
