import pandas as pd
import numpy as np
from scipy.stats import t
from decimal import Decimal, ROUND_HALF_UP
NUM_BOOTS = 100000


ordered_stats = ['log_beta_t_6',
                 'log_cases_per_pop', 
                 'log_deaths_per_pop', 
                 'log_visits',  
                 'delta_log_visits',
                 'treat_on',
                 'log_pop_density',
                 'log_pop',
                 'sh_pop_urban',
                 'summer_tmmx',
                 'winter_tmmx',
                 'sh_pop_commute_auto', 
                 'log_inter_flights', 
                 'r_share', 
                 'sh_pop_age_65plus',
                 'sh_pop_race_black', 
                 'sh_pop_race_white', 
                 'sh_pop_educ_bachelors_plu']

row_order = []
for x in ordered_stats:
    row_order.append(x)
    row_order.append(x + '_std')

col_order = ['all', 'top_half', 'bottom_half', 'half_difference']

def main():
    df = pd.read_csv('output/csa_covariate_means.csv')
    
    final_nums = format_basic(df)
    
    cis = calculate_boots(df)
    
    t_sig = test_significance(final_nums, cis)
    
    fully_formatted = format_human(final_nums)
    
    #adds on columns indicating significance
    t_sig['which'] = ['half_sig']
    reshaped_sig   = t_sig.set_index('which').T
    
    with_sig       = fully_formatted.join(reshaped_sig)
    
    #adds in the nicely formatted confidence intervals
    new_ci = rename_cols_ci(cis)
    with_sig = new_ci.combine_first(with_sig)
    
    #bolds where appropriate
    bolded = bold(with_sig)
    
    #saves in the correct ordere
    final = bolded[col_order].T[row_order].T
    final.to_csv('output/summary_stats.csv', index=False)

#creates the basic structure of std. dev. and means for each column that is considered to be relelvant.
def format_basic(df):
    #finds the means and df for each
    mean_df = pd.DataFrame()
    std_df = pd.DataFrame()
    
    # calculates the overall summary statistics
    mean_df['all'] = df[ordered_stats].mean()
    std_df['all'] = df[ordered_stats].std()
    
    # calculates summary statistics between top and bottom half
    mean_df['top_half']        = df[df.log_beta_t_6 >= df.median()['log_beta_t_6']][ordered_stats].mean()
    std_df['top_half']         = df[df.log_beta_t_6 >= df.median()['log_beta_t_6']][ordered_stats].std()
        
    mean_df['bottom_half']     = df[df.log_beta_t_6 < df.median()['log_beta_t_6']][ordered_stats].mean()
    std_df['bottom_half']      = df[df.log_beta_t_6 < df.median()['log_beta_t_6']][ordered_stats].std()
    
    mean_df['half_difference'] = mean_df['top_half'] - mean_df['bottom_half']
    std_df['half_difference']  = np.sqrt(std_df['bottom_half']**2 + std_df['top_half']**2)
    
    #formats nicely
    joined = mean_df.T.join(std_df.T, rsuffix='_std')
    return joined[row_order].T[col_order]

#does the bootstrap estimates for these values
#resamples from all CSAs and finds the difference in means 
#and calculates the 95% CI from these results
def calculate_boots(df):
    results_med = pd.DataFrame()
    
    #run NUM_BOOTS times
    for i in range(NUM_BOOTS):
        #have seed iteratively different for each time (consistent across runs however)
        resample    = df[ordered_stats].sample(frac=1, replace=True, random_state=i)
        
        top_med     = resample[resample.log_beta_t_6 >= resample['log_beta_t_6'].median()]
        bottom_med  = resample[resample.log_beta_t_6 <= resample['log_beta_t_6'].median()]
        results_med = results_med.append(top_med.mean() - bottom_med.mean(), ignore_index=True)
        
    #reshapes to have well formatted as well as quantiles
    cis = pd.DataFrame()
    cis['median_low']  = results_med.quantile(0.025)
    cis['median_high'] = results_med.quantile(0.975)
    
    cis_rd =  cis.applymap(round_entry)
    cis['median_ci'] = '[' + cis_rd['median_low'] + ', ' + cis_rd['median_high'] + ']'
    
    return cis

#tests significance for the differences
def test_significance(final_nums, cis):
    t_sig = pd.DataFrame()
    
    with_boots = final_nums.join(cis)
    transpose  = with_boots.T
    
    #checks if 0 is outside the bounds of the 95% confidence interval
    for stat in ordered_stats:
        half_diff_sig = (transpose[stat]['median_high'] < 0) or (transpose[stat]['median_low'] > 0)
    
        t_sig[stat]          = [half_diff_sig]
        t_sig[stat + '_std'] = [half_diff_sig]
    
    return t_sig

def round_entry(entry):
    round_to = 3
    decimal_place = round(pow(0.1, round_to), round_to)
    
    if round_to == 0:
        decimal_place = str(int(decimal_place))
    else:
        decimal_place = str(decimal_place)
    rounded_entry = str(Decimal(entry).quantize(Decimal(decimal_place), rounding = ROUND_HALF_UP))
    
    return rounded_entry

#rounds and adds parentheses so they are nearly ready
def format_human(final_nums):
    #rounds all of the numbers to 3 decimal places
    rounded = final_nums.applymap(round_entry)
    #adds parentheses to the std deviations
    parens  = rounded.T[[i + '_std' for i in ordered_stats]].applymap(add_parens)
    return parens.T.combine_first(rounded)

def add_parens(entry):
    return '(' + entry + ')'


#renames the rows in cis such that they match up with '_std' columns where they belong
def rename_cols_ci(cis):
    #replaces the standard deviation with the confidence intervals where relevant.
    rename_dict = {}
    for i in ordered_stats:
        rename_dict[i] = i + '_std'
    
    new_ci = pd.DataFrame()
    new_ci['half_difference']   = cis.T.rename(columns=rename_dict).T['median_ci']
    
    return new_ci

#reformats as strings
#ensures that significance is filled in for all rows
#then adds '\series bold' as a prefix to the significant ones 
def bold(with_sig):
    with_sig['half_sig'] = with_sig['half_sig'].fillna(False)
    with_sig.loc[with_sig['half_sig'] == True, 'half_prefix'] = '\\series bold '
    with_sig['half_prefix'] = with_sig['half_prefix'].fillna('')
    
    with_sig['half_difference'] = with_sig['half_prefix']  + with_sig['half_difference'].astype('str')
    
    return with_sig

if __name__ == '__main__':
    main()

