### SIRD Simulation

main <- function(effect_size){
  # Parameter values follow https://www.sas.upenn.edu/~jesusfv/sird-paper.pdf
  # I deviate by not having the "resolving period". You just go from infectious to recovered.
  params.gamma <- .1   # Speed at which infenctions resolve, e.g., likelihood of resolving in a given day, .2 = 5 days average infection period. 
  params.delta <- .008 # Death rate
  params.R_not_end   <- .95
  params.tau         <- effect_size
  params.R_0 <- 0
  params.D_0 <- 0
  params.N   <- 1
  
  setup <- function(I_0, R_not_start){
    
    # Contact rate
    beta_0  <- R_not_start * params.gamma
    
    # Initial conditions
    S_0 <- params.N - I_0 - params.R_0 - params.D_0
    
    SIRDB_0 <- c(S_0, I_0, params.R_0, params.D_0, beta_0)
    return(SIRDB_0)
  }
  
  update_vals <- function(SIRDB, t, N, gamma, delta, lambda, R_not_start, R_not_end, tau, treated){
    S <- SIRDB[1]
    I <- SIRDB[2]
    R <- SIRDB[3]
    D <- SIRDB[4]
    
    beta <- SIRDB[5]
    
    # Update SIRD
    new_S <- max(S - beta * S * I / N, 0)
    new_I <- I + beta * S * I / N - gamma * I
    new_R <- R + (1 - delta) * gamma * I
    new_D <- D + delta * gamma * I
    
    # Update beta
    new_R_not <- (R_not_start * exp(lambda * t) + R_not_end * (1 - exp(lambda * t))) 
    new_beta  <- new_R_not * gamma * (1 - tau * (t >= treated))
    
    return(c(new_S, new_I, new_R, new_D, new_beta))
  }
  
  
  simulate_county <- function(SIRDB, lambda, R_not_start, treated, pop){
    T         <- 150
    vals      <- matrix(data = NA, nrow = T, ncol = 5)
    vals[1, ] <- SIRDB
    for (t in 1:(T-1)){
      vals[t + 1, ] <- update_vals(vals[t, ], t, 
                                   params.N, 
                                   params.gamma, 
                                   params.delta, 
                                   lambda,
                                   R_not_start, 
                                   params.R_not_end, 
                                   params.tau, treated)
    }
    colnames(vals) <- c("S", "I", "R", "D", "beta")
    vals <- data.frame(vals, stringsAsFactors = F)
    
    vals$C <- rowSums(vals[, 2:4]) * pop
    
    return(vals)
  }
  
  
  # Simulate full data 
  set.seed(2020)
  counties <- 200
  
  # Params that may vary across counties
  base_R_not_start <- 2.4
  base_lambda      <- -.08
  base_I_0         <- .0001
  R_not_dist   <- rnorm(counties, mean = base_R_not_start, sd = .1)
  lambda_dist  <- rnorm(counties, mean = base_lambda     , sd = .01)
  I_0_dist     <- rexp(counties, rate = 1 / base_I_0)
  pop_dist     <- rexp(counties, rate = 1 / 100000)
  init_case    <- I_0_dist * pop_dist
  treated_dist <- rbinom(counties, size = 150, prob = 4/15)
  
  
  DATA <- c()
  for (n in 1:counties){
    SIRDB_0 <- setup(I_0_dist[n], R_not_dist[n])
    
    temp    <- simulate_county(SIRDB_0, lambda_dist[n], R_not_dist[n], treated_dist[n], pop_dist[n])
    
    temp$pop         <- pop_dist[n]
    temp$treated     <- treated_dist[n]
    temp$lambda      <- lambda_dist[n]
    temp$R_not_start <- R_not_dist[n]
    temp$I_0         <- I_0_dist[n]
    temp$t           <- 1:150
    temp$n           <- n
    
    DATA <- rbind(DATA, temp)
  }
  
  return(DATA)
}


DATA <- main(effect_size = .1)
write.csv(DATA, file = "output/simulated_data.csv")

DATA <- main(effect_size = 0)
write.csv(DATA, file = "output/plac_simulated_data.csv")


