program define add_osr2, eclass sortpreserve
	syntax , [fold_var(string) folds(integer 50) rseed(real 9876)]

	******Save current dataset and estimation results, to reload at end*********
	tempname saved_est
	estimates store `saved_est'													//Store previous estimation for later
	
	******Parse stored estimation results***************************************
	local depvar = e(depvar)													//Store dependent variable name
	
	if ("`=e(wtype)'"!=".") {													//Parse weights
		di in red "Warning: Weights have been specified. Regression will be weighted within folds, but weights will be ignored when constructing folds and when averaging R^2 across folds."
		local weight = "[" + e(wtype) + e(wexp) + "]"
	}

	tempvar in_sample resid														
	gen `in_sample' = e(sample)													//Generate indicator for initial estimation sample			

	local cmdline = e(cmdline)													//Parse estimation command
	local found_comma = strpos("`cmdline'", ",")
	if `found_comma' { 															//Split options after comma from pre-comma command
		local opts    = substr("`cmdline'", `found_comma', .)
		local cmdline = substr("`cmdline'", 1, `found_comma'-1)
		local opts    = regexr("`opts'", "vce\(.+\)", "vce(ols)") 				//WLOG, simplify standard errors for speed
	}
	*Remove conditional statements and weights, now handled by `in_sample' and `weight'
	local cmdline = regexr("`cmdline'", " if .+$", "") 							//Remove if
	local cmdline = regexr("`cmdline'", " in .+$", "") 							//Remove in
	local cmdline = regexr("`cmdline'", "\[.+\]", "")  							//Remove weights
	
	******Specify fold variable*************************************************
	if "`fold_var'"=="" tempvar fold_var										//Use temp variable if no fold_var specified
	cap confirm var `fold_var' 													//Check for existing fold variable
	if _rc {			       													//If missing, create variable
		local rseed_opt = cond(`rseed'==9876, "", "rseed(`rseed')")
		splitsample if `in_sample'==1, nsplit(`folds') gen(`fold_var') `rseed_opt' //Create fold_var
	}
	else confirm numeric var `fold_var' 										//If variable found, confirm it's numeric
	assert !missing(`fold_var') if `in_sample'==1
	
	******Construct MSE across CV folds*****************************************
	tempvar resid_sq depvar_mean depvar_sqer
	qui levelsof `fold_var' if `in_sample'==1, local(fold_ns)
	local mse  = 0																//Sum together mean squared error from each fold in this local
	local rmse = 0																//Sum together root mean squared error from each fold in this local
	foreach fold of local fold_ns {												//Loop through folds
		qui `cmdline' if `in_sample'==1 & `fold_var'!= `fold' `opts'			//Run estimation, leaving out given fold
		qui predict double `resid' if `in_sample'==1 & `fold_var'==`fold', resid //Get out of sample residuals
		qui gen double `resid_sq' = `resid'^2 if `in_sample'==1 & `fold_var'==`fold' //Get squared residuals
		summ `resid_sq' if `in_sample'==1 & `fold_var'==`fold' `weight', meanonly  //Get MSE within given fold
		local mse  = `mse'  +      r(mean)
		local rmse = `rmse' + sqrt(r(mean))
		drop `resid' `resid_sq'
	}
	local mse  = `mse'  / `: word count `fold_ns''								//Average MSE across left-out folds
	local rmse = `rmse' / `: word count `fold_ns''								//Average MSE across left-out folds
	
	******Prepare y^2***********************************************************
	qui egen double `depvar_mean' = mean(`depvar')             if `in_sample'==1 //Calculate complete sample y_bar 
	qui gen double `depvar_sqer'  = (`depvar'-`depvar_mean')^2 if `in_sample'==1 //Calculate complete sample (y-y_bar)^2
	summ `depvar_sqer' if `in_sample'==1, meanonly
	local osr2 = 1 - `mse'/(r(mean))											//Convert MSE to out-of-sample R^2
	
	******Restore original data and estimation, add e(osr2)*********************
	qui estimates restore `saved_est'											//Bring back original e() values
	estimates drop `saved_est'													//Drop temporary saved estimation
	ereturn scalar rmse = `rmse'												//Add e(rmse)
	ereturn scalar mse = `mse'													//Add e(mse)
	ereturn scalar osr2 = `osr2'												//Add e(osr2)
	
end
