/***
	Program: eventplot.ado
	Purpose: Plots coefficients from an event study specification
	Author: Benny Goldman (2020-4-4)
***/

program define eventplot

	version 16.1
	syntax [anything] [if] [aweight fweight], ///
	treatvar(varname) eventtime(varname) absorb(varlist) ///
	[leads(integer 10) lags(integer 10) omit(integer 1) robust ///
	clustervar(varname) title(string) addplot(string) storecoef addcol(string) ///
	output(string) xtitle(string) *]
	
	* Use if condition
	marksample touse
	
	* Assign color to additional plot
	if "`addcol'"=="" {
		local col "gs10%60"
	}
	if "`addcol'"!="" {
		local col "`addcol'"
	}
	
	* Create convenient weight local
	if ("`weight'"!="") local wt [`weight'`exp']

	* Set xtitle default
	if (`"`xtitle'"'=="") local xtitle = "Days Relative to Order"
	
	* Leading indicators
	quietly forvalues t=0/`leads' {
		g byte lead_`t'=`eventtime'==-`t' & `treatvar'==1
		* Binning up
		if `t'==`leads' {
			replace lead_`t'=`eventtime'<=-`t' & `treatvar'==1
		}
	}

	* Lagging indicators
	quietly forvalues t=1/`lags' {
		g byte lag_`t'=`eventtime'==`t' & `treatvar'==1
		if `t'==`lags' {
			replace lag_`t'=`eventtime'>=`t' & `treatvar'==1
		}
	}

	* Drop the omitted leading variable
	quietly drop lead_`omit'
	
	* Standard error options
	if "`robust'"!="" {
		local se_opts "vce(robust)"
	}
	if "`clustervar'"!="" {
		local se_opts "vce(cluster `clustervar')"
	}
	if "`clustervar'"!="" & "`robust'"!=""{
		di as error "Cannot specify both robust and clustering"
		exit
	}
	
	* Run estimation
	reghdfe `anything' `wt' if `touse'==1, absorb(`absorb') `se_opts'

	* Create empty variables to store results
	* Do not include the binned up and binned down coefficients
	quietly {
		g event_time=_n-`leads'-1 if _n<=`leads'+`lags'+1
		g coef=.
		g se=.

		* Fill in the leading coefficients and standard errors
		foreach t of numlist 0/`=`omit'-1' `=`omit'+1'/`=`leads'-1' {
				replace coef=_b[lead_`t'] if event_time==-`t'
				replace se=_se[lead_`t'] if event_time==-`t'
		}

		* Fill in omitted row
		replace coef=0 if event_time==-`omit'
		replace se=0 if event_time==-`omit'

		* Fill in the lagging coefficients and standard errors
		foreach t of numlist 1/`=`lags'-1' {
				replace coef=_b[lag_`t'] if event_time==`t'
				replace se=_se[lag_`t'] if event_time==`t'
		}

		* Make top and bottom of standard error bars
		g top=coef+1.96*se
		g bot=coef-1.96*se
	}

	
	* Built plot
	if "`output'"!="" {
		if "`addplot'"!="" {
			twoway ///
				scatter coef_`addplot' event_time, mcolor("`col'") || ///
				rcap top_`addplot' bot_`addplot' event_time, lpattern(solid) lcolor("`col'") || ///
				scatter coef event_time, mcolor("scheme p1") || ///
				rcap top bot event_time, lpattern(solid) lcolor("scheme p1") ///
				`options' ///
				xtitle(`"`xtitle'"') ///
				xline(0, lpattern(dash) lcolor("scheme p2")) ///
				ytitle("Coefficient Estimate") ylabel(, gmax format(%03.2f)) ///
				title("`title'", size(*0.85))	
		}
		else {
			twoway ///
				scatter coef event_time || ///
				rcap top bot event_time, lpattern(solid) lcolor("scheme p1") ///
				`options' ///
				xtitle("Days Relative to Order") ///
				xline(0, lpattern(dash) lcolor("scheme p2")) ///
				ytitle("Coefficient Estimate") ylabel(, gmax format(%03.2f)) ///
				title("`title'", size(*0.85))
		}
		
		* Output graph
		graph export "`output'.pdf", replace
		
		* Output txt file
		preserve
		keep event_time coef se
		drop if mi(event_time)
		export delimited using "`output'.txt", delimiter(tab) replace
		restore
	
	}
	

	* Drop variables created throughout the program
	drop lead* lag* 
	if "`storecoef'"=="" {
		drop event_time se coef top bot
	}

end
